$(function() {
	/*
	 * Teksten op de pagina vullen afhankelijk van platform
	 */
	$('#backupPage').on('pagebeforeshow', function() {
		hideAllControls();
	});

	$('#backupPage').on(
			'pageshow',
			function() {
				var account = accountController.getOrCreateAccount();
				var loggedIn = false;

				if (account != null) {
					loggedIn = account.loggedIn;
				}

				var backupText = "";
				if (!loggedIn) {
					backupText = jQuery.i18n.prop('i18n_not_loggedin');
					hideAllControls();
				} else {
					/*
					 * Check if user accepted license already
					 */
					serviceController.getRequest(EP_SYNCHRONISE_ENDPOINT, EP_LICENSE_ACCEPTED_GET,
							getOkWithLicenseSuccess, licenseErrors);
				}

				$('#backupText').html(backupText);
			});

	function getOkWithLicenseSuccess(userAcceptedLicense) {
		console.log("get OK with license:  " + userAcceptedLicense);
		if (userAcceptedLicense) {
			backupText = jQuery.i18n.prop('i18n_backup_from_server');
			showAccountControls();
		} else {
			var settings = getSettings();
			serviceController.getRequest(EP_SYNCHRONISE_ENDPOINT, EP_LICENSE_GET + settings.language,
					getLicenseSuccess, licenseErrors);
		}
	}

	function getLicenseSuccess(licenseText) {
		$('#licenseText').html(licenseText);
		showLicenseControls();
	}

	function licenseErrors(response) {
		if(response.statusText === "Unauthorized") {
			phoneAlert(jQuery.i18n.prop('i18n_account_login_error'));
		} else {
			$('#licenseText').html(licenseText).show();
		}
	}

	function setOkWithLicenseSuccess() {
		backupText = jQuery.i18n.prop('i18n_backup_from_server');
		showAccountControls();

		/*
		 * Door een raar issue met de jquery.scrollview wordt er na het hiden
		 * van de lange licentie tekst een veel te grote -webkit-translate3d
		 * gedaan op de div waarbinnen alle content staat. Deze snelle fix/hack
		 * haalt de style property weer van de div af waardoor het scherm gewoon
		 * zichtbaar wordt.
		 */
		$('#backupPage .ui-scrollview-view').attr('style', '');
	}
	/*
	 * Accept license agreement knop
	 */
	$('#licenseAcceptButton').click(function() {
		serviceController.getRequest(EP_SYNCHRONISE_ENDPOINT, EP_LICENSE_ACCEPTED_SET, setOkWithLicenseSuccess, null);
	});

	/*
	 * Synchronise with server knop
	 */
	$('#syncServerButton').click(function() {
		sendBackupToServer();
	});

	/*
	 * Gegevens terugzetten vanaf server knop
	 */
	$('#restoreServerButton').click(function() {
		restoreBackupFromServer();
	});
});

/*-===============================
 * GENERAL FUNCTIONS
 *================================*/

function showLicenseControls() {
	hideAllControls();
	$('#licenseText').show();
	$('#licenseAcceptButton').show();
}

function showAccountControls() {
	hideAllControls();
	$('#syncServerButton').show();
	$('#restoreServerButton').show();
	$('#getBehandelschemaButton').show();
}

function hideAllControls() {
	$('#licenseText').hide();
	$('#licenseAcceptButton').hide();
	$('#syncServerButton').hide();
	$('#restoreServerButton').hide();
	$('#getBehandelschemaButton').hide();

}

function createBackupObject(decrypt) {
	var backupArray = [];
	for ( var i = 0; i < window.localStorage.length; i++) {
		var currentKey = window.localStorage.key(i);
		if (decrypt != undefined && decrypt == true) {
			try {
				backupArray.push(new Array(currentKey, aesDecrypt(window.localStorage.getItem(currentKey))));
			} catch (err) {
				console.log("Backup.js: createBackupObject - unable to decrypt key: " + currentKey);
			}
		} else {
			backupArray.push(new Array(currentKey, window.localStorage.getItem(currentKey)));
		}
	}

	return backupArray;
}

/*-===============================
 * SYNCHRONIZE WITH SERVER
 *================================*/

function sendBackupToServer() {

	var convertType = function(name) {
		switch (name) {
		case getEnumByValue(EP_ALGEMEEN_TYPE, 0):
			return "NOTITIE";
		case getEnumByValue(EP_ALGEMEEN_TYPE, 1):
			return "STEMMING";
		case getEnumByValue(EP_ALGEMEEN_TYPE, 2):
			return "ZWANGERSCHAP";
		case getEnumByValue(EP_ALGEMEEN_TYPE, 3):
			return "MENSTRUATIECYCLUS";
		case getEnumByValue(EP_ALGEMEEN_TYPE, 4):
			return "GENOTSMIDDELEN";
		case getEnumByValue(EP_ALGEMEEN_TYPE, 5):
			return "ZIEKTE";
		case getEnumByValue(EP_ALGEMEEN_TYPE, 6):
			return "SLAAPRITME";
		case getEnumByValue(EP_ALGEMEEN_TYPE, 7):
			return "BIORITME";
		}
	};

	var convertMedicijnEenheid = function(enumPosition) {
		switch (enumPosition) {
		case "0":
			return "STUKS";
		case "1":
			return "GRAM";
		case "2":
			return "MILLILITER";
		case "3":
			return "MILLIGRAM";
		}
	};

	var convertAfwijkendeReden = function(reden) {
		switch (reden) {
		case "extra":
			return "EXTRA_INGENOMEN";
		case "missed":
			return "GEMIST";
		case "later":
			return "LATER_INGENOMEN";
		}
	};

	var parseMedication = function(b) {
		// Maak voor elk dagdeel een inname moment aan
		var dayParts = b.dayparts;

		var result = {
			"remoteId" : b.id,
			"medicijnNaam" : b.drugName,
			"eenheid" : convertMedicijnEenheid(b.unit),
			"herinnering" : b.sendNotification,
			"innameMomenten" : [],
		};

		for ( var i = 0; i < dayParts.length; i++) {
			var tijdStip = new Date();
			var timeArr = dayParts[i].time.split(":");
			if (timeArr[1] != undefined) {
				tijdStip.setHours(timeArr[0]);
				tijdStip.setMinutes(timeArr[1]);
			}
			var innameMoment = {
				"hoeveelheid" : dayParts[i].quantity,
				"innameTijdstip" : tijdStip.getTime(),
			};

			result.innameMomenten.push(innameMoment);
		}

		return result;
	};

	var parseContact = function(b) {
		var result = {
			"remoteId" : b.id,
			"naam" : b.name,
			"relatie" : b.role,
			"emailAdres" : b.email,
			"telefoonNummer" : b.phoneNumber,
			"belContact" : b.isQuickDial,
			"volgMijContact" : b.isFollower,
		};

		return result;
	};

	var parseEvent = function(b) {
		var eventType = b.eventType;
		var specializedEvent = {};

		//29-03-2013 datafix voor aanval registratie via alarm functie die geen einddatum hebben
		if(eventType.name == "Aanval" && b.eindDateTime == undefined) {
			// Datums zo instellen dat storeJSON goed gaat
			b.eindDateTime = Date.parseExact(b.startDateTime, "yyyy-MM-ddTHH:mm:ss").addMinutes(b.duration);
			b.startDateTime =  Date.parseExact(b.startDateTime, "yyyy-MM-ddTHH:mm:ss");
			storeJSON(b, b.id); // ervoor zorgen dat dit object niet weer gefixt hoeft te worden.
			
			// Datums weer zo instellen dat het 'generalEvent' stukje goed blijft gaan
			b.startDateTime = b.startDateTime.toString("yyyy-MM-ddTHH:mm:ss");
			b.eindDateTime = b.eindDateTime.toString("yyyy-MM-ddTHH:mm:ss");
		} // end fix
		
		var generalEvent = {
			"remoteId" : b.id,
			"startDateTime" : Date.parseExact(b.startDateTime, "yyyy-MM-ddTHH:mm:ss").getTime(),
			"eindDateTime" : Date.parseExact(b.eindDateTime, "yyyy-MM-ddTHH:mm:ss").getTime(),
			"notitie" : b.notes,
		};

		if (eventType.custom == true) {
			/* EIGEN GEBEURTENIS */
			specializedEvent = {
				"type" : "EppyEigenGebeurtenis",
				"titel" : eventType.eigenTitel
			};
		} else if (eventType.name == "Bijwerking") {
			/* BIJWERKING */
			specializedEvent = {
				"type" : "EppyBijwerking",
				"bijwerking" : b.name,
				"intensiteit" : b.intensity,
			};
		} else if (eventType.name == "IncidenteleMedicatie") {
			/* INCIDENTELE MEDICATIE */
			specializedEvent = {
				"type" : "EppyAfwijkendeMedicatie",
				"medicijnNaam" : b.drugName,
				"hoeveelheid" : b.quantity,
				"eenheid" : convertMedicijnEenheid("" + b.unit),
				"reden" : "INCIDENTELE_MEDICATIE"
			};
		} else if (eventType.custom == false && eventType.name.match("^algemeen_")) {
			/* ALGEMENE GEBEURTENIS */
			specializedEvent = {
				"type" : "EppyAlgemeen",
				"eventType" : convertType(eventType.name),
			};
		} else if (eventType.name == "Aanval") {
			/* AANVAL */
			specializedEvent = {
				"type" : "EppyAanval",
				"aantalMinuten" : b.duration,
				"aantalAanvallen" : b.strokeCount,
				"aanvalType" : b.strokeType,
				"intensiteit" : b.intensity
			};
		} else if (eventType.name == "AfwijkendMedicijngebruik") {
			/* AFWIJKENDE MEDICATIE */
			// var overrideStartDateTime = new Date(b.startDateTime);
			var overrideStartDateTime = Date.parse(b.startDateTime, "yyyy-MM-ddTHH:mm:ss");
			var timeArr = b.time.split(":");

			if (timeArr[1] != undefined) {
				overrideStartDateTime.setHours(timeArr[0]);
				overrideStartDateTime.setMinutes(timeArr[1]);
				overrideStartDateTime.setSeconds(0);
			}

			specializedEvent = {
				"type" : "EppyAfwijkendeMedicatie",
				"medicijnNaam" : b.drugName,
				"hoeveelheid" : b.quantity,
				"eenheid" : convertMedicijnEenheid("" + b.unit),
				"reden" : convertAfwijkendeReden(b.reason),
				"startDateTime" : overrideStartDateTime.getTime(),
			};
		} else {
			return null;
		}

		// Enrich general event and return
		for ( var attrname in specializedEvent) {
			generalEvent[attrname] = specializedEvent[attrname];
		}
		return generalEvent;
	}; /* end parseEvent() */

	var eventArray = [];
	var contactArray = [];
	var medicationArray = [];

	/* Over all localstorage key/values heen lopen */
	for ( var i = 0; i < window.localStorage.length; i++) {
		var currentKey = window.localStorage.key(i);
		var currentValue = getJSON(currentKey);

		if (currentValue != null) {

			for ( var j = 0; j < currentValue.length; j++) {
				
				var b = currentValue[j];

				/*
				 * In dit gedeelte bepalen we welk type over de lijn moet en
				 * vormen we dit om naar de juiste representatie zoals de REST
				 * webservice die verwacht.
				 */
				try {
					
					if (b.objectType == "epEvent") {
						var event = parseEvent(b);
						if (event != null)
							eventArray.push(event);
					} else if (b.objectType == "epRecurringMedicine") {
						var medication = parseMedication(b);
						if (medication != null)
							medicationArray.push(medication);
					} else if (b.objectType == "epContact") {
						var contact = parseContact(b);
						if (contact != null)
							contactArray.push(contact);
					} else {
						console.log("Backup to server: Kan dit eventtype niet verwerken: " + JSON.stringify(currentValue[j]));
					}
				} catch (err) {
					console.log("Exception tijdens synchroniseren:" + err + " Dit object wordt overgeslagen: " + JSON.stringify(currentValue[j]));
				}
			}
		}
	}

	var account = accountController.getOrCreateAccount();
	var gebruikersNaam = "";
	if (account != null && account.username != undefined) {
		gebruikersNaam = account.username;
	} else {
		console.log("Backup to server: Fout bij bepalen account");
	}

	var settings = getSettings();
	var berichten = false;
	var anoniem = false;
	if (settings != null) {
		berichten = settings.adminMail;
		anoniem = settings.useMyData;
	} else {
		console.log("Backup to server: Fout bij ophalen settings");
	}

	var result = {
		"versie" : EP_BUILD,
		"wilBerichtenOntvangen" : berichten,
		"wilAnoniemGegevensDelen" : anoniem,
		"events" : eventArray,
		"contacts" : contactArray,
		"medication" : medicationArray
	};

	console.log(JSON.stringify(result));

	serviceController.sendRequest(EP_SYNCHRONISE_ENDPOINT, EP_SYNCHRONISE_ALL, result, function(data) {
		if (data && data.status === "ERROR") {
			phoneAlert($.i18n.prop('i18n_sync_error') + " - " + data.message);
		} else {
			phoneAlert($.i18n.prop('i18n_sync_success'));
		}
	}, function(jqXHR, textStatus, errorThrown) {
		phoneAlert($.i18n.prop('i18n_sync_error') + " - " + errorThrown);
	});
}

/*-===============================
 * RESTORE BACKUP FROM SERVER
 *================================*/

function restoreBackupFromServer() {

	var restoreData = {

		restore : function() {
			// Haal account gegevens op
			var account = accountController.getOrCreateAccount();
			var gebruikersNaam = "";
			if (account != null && account.username != undefined) {
				gebruikersNaam = account.username;

			}

			var jsonRequest = {};

			// Doe een verzoek om de backup data naar de server
			serviceController.getRequest(EP_SYNCHRONISE_ENDPOINT, EP_GET_ALL, restoreData.verwerkServerData,
					restoreData.restoreFailedCB);
		},

		restoreFailedCB : function(jqXHR, textStatus, errorThrown) {
			phoneAlert($.i18n.prop('i18n_sync_error') + " - " + errorThrown);
		},

		/*
		 * Als het verzoek om de backup data succesvol is, hier verwerken
		 */
		verwerkServerData : function(response) {
			console.log(response);

			if (response.status == "SUCCESS") {
				var serverData = response.syncAllObject;

				if (serverData == null) {
					phoneAlert(jQuery.i18n.prop('i18n_backup_error'));
				} else {
					// Stap 1: Localstorage leegmaken
					restoreData.deleteLocalStorage();

					var contactResult = true;
					var eventResult = true;
					var medicationResult = true;

					// Stap 2: Herstel gegevens die via de server zijn ontvangen
					if (serverData.contacts != null) {
						// Stap 2a: herstel contacten
						try {
							restoreData.restoreContacts(serverData.contacts);
						} catch (err) {
							console.log("Contact restore error! catch.");
							contactResult = false;
						}
					}
					if (serverData.events != null) {
						// Stap 2b: herstel gebeurtenissen
						try {
							restoreData.restoreEvents(serverData.events);
						} catch (err) {
							console.log("Events restore error! catch.");
							eventResult = false;
						}
					}
					if (serverData.medication != null) {
						// Stap 2c: herstel medicatie
						try {
							restoreData.restoreMedication(serverData.medication);
						} catch (err) {
							console.log("Medicatie restore error! catch.");
							medicationResult = false;
						}
					}

					// Stap 2d: zet gebruikersvoorkeuren terug
					var settings = getSettings();
					settings.useMyData = serverData.wilAnoniemGegevensDelen;
					settings.adminMail = serverData.wilBerichtenOntvangen;

					phoneAlert(jQuery.i18n.prop('i18n_backup_restore_success_nopin'));
				}
			} else {
				if (response.status === "ERROR") {
					phoneAlert(jQuery.i18n.prop('i18n_backup_error') + response.message);
					return;
				}
				phoneAlert(jQuery.i18n.prop('i18n_backup_error'));
			}
		},

		/* Terugzetten van alle contact personen */
		restoreContacts : function(contactArray) {
			for ( var i = 0; i < contactArray.length; i++) {
				var contact = contactArray[i];

				var c = new epContact(contact.naam);
				c.phoneNumber = contact.telefoonNummer;
				c.email = contact.emailAdres;
				c.role = contact.relatie;
				c.isQuickDial = contact.belContact;
				c.isFollower = contact.volgMijContact;

				storeJSON(c);
			}
		},

		convertEenheid : function(eenheid) {
			if (eenheid == "stuks") {
				return 0;
			} else if (eenheid == "gram") {
				return 1;
			} else if (eenheid == "milliliter") {
				return 2;
			} else if (eenheid == "milligram") {
				return 3;
			}
		},

		convertAfwijkendeReden : function(reden) {
			switch (reden) {
			case "extra_ingenomen":
				return "extra";
			case "gemist":
				return "missed";
			case "later_ingenomen":
				return "later";
			}
		},

		/* Terugzetten van alle gebeurtenissen */
		restoreEvents : function(eventArray) {

			for ( var i = 0; i < eventArray.length; i++) {
				var event = eventArray[i];
				var eventStartDate = (new Date(event.startDateTime));

				var generalEvent = {
					"id" : event.remoteId,
					"startDateTime" : eventStartDate,
					"eindDateTime" : (new Date(event.eindDateTime)),
					"notes" : event.notitie,
				};

				var result = {};

				if (event.type == "EppyAanval") {
					/* AANVAL */
					result = new epEventStroke();
					result.intensity = event.intensiteit;
					result.strokeCount = event.aantalAanvallen;
					result.duration = event.aantalMinuten;
					result.strokeType = event.aanvalType;

					// Herstellen van aanvalTypes snelkieslijst
					var aanvalTypes = getJSON("epAanvalType");
					var aanvalTypeGevonden = false;
					if (aanvalTypes != null) {
						for ( var j = 0; j < aanvalTypes.length; j++) {
							var aanvalType = aanvalTypes[j];
							if (aanvalType.typeName == event.aanvalType) {
								aanvalTypeGevonden = true;
							}
						}
					}
					if (!aanvalTypeGevonden) {
						var aanvalType = new epAanvalType();
						aanvalType.typeName = event.aanvalType;

						storeJSON(aanvalType);
					}

				} else if (event.type == "EppyAlgemeen") {
					/* ALGEMENE GEBEURTENIS */
					var eventType = null;

					if (event.eventType == "notitie") {
						eventType = getEnumByValue(EP_ALGEMEEN_TYPE, 0);
					} else if (event.eventType == "stemming") {
						eventType = getEnumByValue(EP_ALGEMEEN_TYPE, 1);
					} else if (event.eventType == "zwangerschap") {
						eventType = getEnumByValue(EP_ALGEMEEN_TYPE, 2);
					} else if (event.eventType == "menstruatiecyclus") {
						eventType = getEnumByValue(EP_ALGEMEEN_TYPE, 3);
					} else if (event.eventType == "genotsmiddelen") {
						eventType = getEnumByValue(EP_ALGEMEEN_TYPE, 4);
					} else if (event.eventType == "ziekte") {
						eventType = getEnumByValue(EP_ALGEMEEN_TYPE, 5);
					} else if (event.eventType == "slaapritme") {
						eventType = getEnumByValue(EP_ALGEMEEN_TYPE, 6);
					} else if (event.eventType == "bioritme") {
						eventType = getEnumByValue(EP_ALGEMEEN_TYPE, 7);
					}
					result = new epEventAlgemeen(eventStartDate, eventType);
				} else if (event.type == "EppyAfwijkendeMedicatie") {
					if (event.reden == "incidentele_medicatie") {
						/* INCIDENTEEL MEDICIJNGEBRUIK */
						result = new epEventIncidentDrug(eventStartDate, event.medicijnNaam);
						result.quantity = event.hoeveelheid;
						result.unit = restoreData.convertEenheid(event.eenheid);
					} else {
						/* AFWIJKEND MEDICIJNGEBRUIK */
						result = new epEventCustomMedicine(eventStartDate, event.medicijnNaam);
						result.quantity = event.hoeveelheid;
						result.unit = restoreData.convertEenheid(event.eenheid);
						result.time = eventStartDate.toString("HH:mm");
						result.reason = restoreData.convertAfwijkendeReden(event.reden);
					}
				} else if (event.type == "EppyBijwerking") {
					/* BIJWERKING */
					result = new epEventSideEffect(eventStartDate, event.bijwerking);
					result.intensity = event.intensiteit;
				} else if (event.eventType = "EppyEigenGebeurtenis") {
					/* EIGEN GEBEURTENIS */
					var et = new epEventType("CustomEvent");
					et.custom = true;
					et.eigenTitel = event.titel;

					result = new epEventCustom(eventStartDate, et);
				}

				// Enrich general event and return
				for ( var attrname in generalEvent) {
					result[attrname] = generalEvent[attrname];
				}

				storeJSON(result);
			}
		},

		/* Terugzetten van persoonlijke behandelschema */
		restoreMedication : function(medicationArray) {
			console.log("medication: " + medicationArray);

			for ( var i = 0; i < medicationArray.length; i++) {
				var med = medicationArray[i];

				var medicijn = new epRecurringMedicine(med.medicijnNaam);
				medicijn.id = med.remoteId;
				medicijn.drugName = med.medicijnNaam;
				medicijn.unit = restoreData.convertEenheid(med.eenheid);
				medicijn.sendNotification = false;

				var medDagdelen = med.innameMomenten;
				var dayParts = [];

				// Maximaal 4 dagdelen terugzetten
				for ( var j = 0; j < medDagdelen.length && j <= 4; j++) {
					var medDagdeel = medDagdelen[j];
					var time = new Date(medDagdeel.innameTijdstip);
					var min = time.getMinutes();
					if (min < 10) {
						min = "0" + min;
					}

					var dayPart = {
						"daypart" : medDagdeel.innameTijdstipNaam,
						"quantity" : medDagdeel.hoeveelheid,
						"time" : time.getHours() + ":" + min,
					};

					dayParts.push(dayPart);
				}

				medicijn.dayparts = dayParts;

				storeJSON(medicijn);
			}
		},

		/**
		 * Verwijdert alle localstorage items behalve epSettings en epAccount
		 */
		deleteLocalStorage : function() {
			/* Over all localstorage key/values heen lopen */
			var settings = getSettings();
			var account = accountController.getOrCreateAccount();

			window.localStorage.clear();

			// Aanval types herstellen
			createStandaardAanval();
			// Settings herstellen
			storeJSON(settings);
			// Account gegevens herstellen
			storeJSON(account);
			// Multiday cache flushen
			refreshMultiDayCache();
		}
	}; // end restoreData

	// Bevestiging vragen en doorgaan of afbreken.
	phoneConfirm(jQuery.i18n.prop('i18n_sync_server_warning'), restoreData.restore, function() {
		return;
	});

}
