EP_REPORT_SYMBOLS = [ "triangle", "circle", "square" ];
var EP_REPORT_XAXIS_TICKS_MAX = 10;

var epChart = function() {

	this.plotData = [];
	// callbackFunctions: [],
	this.options = {
		points : {
			show : true,
			fill : true,
			radius : 4,
			type : "o"
		},
		xaxis : {
			mode : "time",
			// min : Date.UTC(2011, 11, 1, 0, 0, 0),
			// max : Date.UTC(2011, 11, 30, 0, 0, 0),
			//timeformat : "%d-%m", // display only the day number
			tickSize : [ 1, "day" ], // 2 *days* interval
//			tickLength : 0, //werkt pas vanaf fl0t 0.7!
		// disable the vertical lines in the grid
			tickFormatter : function (val, axis) {
			    var d = new Date(val);
			    return d.toString('dd-MMM');
			  }

		},
		yaxis : {
			min : 0,
			tickDecimals : 0,
		},
		xaxes : [ {position : "bottom" } ],
		y2axis : { position : "right", min : 0, tickDecimals : 2, tickSize : 0.25 },
//		yaxes : [ { }, { position : "right", min : 0, tickDecimals : 2 } ], //dit is alleen voor de nieuwe versie van flot...
		grid : {
			canvasText : {
				backgroundColor : {
					colors : [ "#000000", "#eee" ]
				},
				show : true,
				font : "sans 8px",
			}
		},
		

		
	};
	
	this.setXaxisRange = function(beginDate, endDate) {
		this.options.xaxis.min = beginDate;
		this.options.xaxis.max = endDate;
		var totalDays = getDifferenceInDays(beginDate, endDate);
		var tickInterval = totalDays > EP_REPORT_XAXIS_TICKS_MAX ? Math.ceil(totalDays / EP_REPORT_XAXIS_TICKS_MAX) : 1;
		if (totalDays > 62) {
			//label veranderen naar maanden als er een grote range is gekozen.
//			this.options.xaxis.timeformat = "%m-%y"; 	
			this.options.xaxis.tickFormatter = function (val, axis) {
			    var d = new Date(val);
			    return d.toString('MMM-yy');
			  };

		}
		this.options.xaxis.tickSize = [ tickInterval, "day" ];
	};

	this.saveAsPNG = function(placeholderID, targetID) {
		var divobj = document.getElementById(placeholderID);
		var oImg = Canvas2Image.saveAsJPEG(divobj.childNodes[0], true);

		if (!oImg) {
			alert("Sorry, this browser is not capable of saving PNG files!");
			return false;
		}

		oImg.id = "canvasimage";
		document.getElementById(targetID).removeChild(document.getElementById(targetID).childNodes[0]);
		document.getElementById(targetID).appendChild(oImg);
	};

	/**
	 * Toevoegen van events. label = vertaalde naam van event;
	 */
	this.addData = function(data,label,color) {
		var chartData = {
			label : jQuery.i18n.prop(label), // todo: andere naam
			data : data,

			bars : {
				show : true,
				color : color,
				fill : true,
				// fillColor : "rgba(114, 191, 68, 0.8)",
				barWidth : (12 * 60 * 60 * 1000), // milliseconds
				align : "center"
			}
		};

		this.plotData.push(chartData);
	};

	/**
	 * Toevoegen van medicijndata (label = naam van medicijn)
	 */
	this.addMedicineData = function(drugName, medData) {
		var meds = {
			label : drugName,
			data : medData,
			yaxis : 2,
			align : "center",
			lines : {
				lineWidth: 0.7,
				show : true,
			},
			points : {
				show : true,
			// symbol : EP_REPORT_SYMBOLS[this.plotData.length - 1]
			}
		};

		this.plotData.push(meds);
	};

	this.addMedicines = function(dateBegin, dateEnd) {
		var totalDays = getDifferenceInDays(dateBegin, dateEnd);

		var meds = mijnMedicatieController.getAll();
		for ( var m in meds) {

			// var totalMedQuantity = 0;

			var day = new Date(dateBegin.getTime());
			var plotPoints = [];
			for ( var d = 0; d < totalDays; d++) {

				if (meds[m].dayparts) {
					for ( var part in meds[m].dayparts) {
						if (meds[m].dayparts[part]) {

							var dateString = day.toString("yyyy-MM-dd ") + meds[m].dayparts[part].time;
							var point = [ Date.parse(dateString), meds[m].dayparts[part].quantity ];
							plotPoints.push(point);
							// totalMedQuantity = meds[m].dayparts[d].quantity;
						}
					}
				}

				day.addDays(1);
			}

			this.addMedicineData(meds[m].drugName, plotPoints);
		}
	};

	/**
	 * Toevoegen van de lijst met gebeurtenissen aan de plotdata
	 * 
	 * @param eventName
	 * @param eventData
	 */
	this.addEventData = function(eventName, eventData) {
		var events = {
			label : jQuery.i18n.prop(eventName),
			data : eventData,
			yaxis : 1,
			align : "center",
			points : {
				show : true,
			// symbol : EP_REPORT_SYMBOLS[this.plotData.length - 1]
			}
		};

		this.plotData.push(events);
	};

	/**
	 * Draw the plot in the targeted #ID
	 * 
	 * @param plotArea
	 */
	this.drawIn = function(placeholder) {
		var plotArea = $(placeholder);
		plotArea.empty();
		if (this.plotData.length > 0) {
			// Automatically fit the graph to the screen
			plotArea.width($(window).width() - 70);
			plotArea.height($(window).height() - 60);
			var plot = $.plot(plotArea, this.plotData, this.options);
		} else {
			//no data found
			plotArea.html(jQuery.i18n.prop('i18n_chart_not_enough_data'));
		}
		
	};

};

function getRnd(maxVal) {
	return Math.floor(Math.random() * maxVal);
}

/**
 * bouwt de array met plotpunten op voor een eventtype. Elk eventtype staat voor 1 punt, allen aanvallen kunnen meerdere in 1 event hebben.
 * @param eventTypeName
 * @param dateBegin
 * @param dateEnd
 * @returns {Array}
 */
function getEventPlotPoints(eventTypeName, dateBegin, dateEnd) {
	var totalDays = getDifferenceInDays(dateBegin, dateEnd);

	var plotPoint = [];
	var plotPoints = [];
	var day = new Date(dateBegin.getTime());
	var eventDays = [];

	for ( var d = 0; d < totalDays; d++) {
		var events = getJSON(day.toDateString());
		if (events != null) {
			for ( var e in events) {
				if (events[e].eventType.name == eventTypeName) {
					
					var aantalEvents = (eventTypeName == 'Aanval') ? events[e].strokeCount : 1;
					
					if (eventDays[day.toDateString()]) {
						eventDays[day.toDateString()] += 1;
					} else {
						eventDays[day.toDateString()] = 1;
					}
				}
			}
		}
		day.addDays(1);
	}

	for ( var s in eventDays) {
		var date = Date.parse(s);
		plotPoint = [ date, eventDays[s] ];
		plotPoints.push(plotPoint);
	}

	return plotPoints;
}

var chart;

/**
 * Selecteert de range uit de reportpagina en zet de header, vult de data voor elk geselecteerd event, en tekent de grafiek in de placeholder.
 */
function drawPlot() {

	var selectedRange = $('#reportRange > option:selected').val();
	/*
	 * Einddatum is altijd vandaag, tenzij de rangeselector is gekozen.
	 * Begindatum wordt bepaald aan de hand van de gekozen range (en einddatum).
	 * Tenzij de rangeselector is gekozen.
	 */
	var endDate = (selectedRange != "range") ? new Date() : Date.parse($('#reportEindDatum').val());
	var beginDate = (selectedRange != "range") ? determineBeginDate(selectedRange, endDate) : Date.parse($('#reportBeginDatum').val());

	$('#plotPageHeaderTitle').html(beginDate.toString('dd/MM/yy - ') + endDate.toString('dd/MM/yy'));
	
	// Markup the data for the graph
	chart = new epChart();
	if (isSelected("Aanval")) {
		var seizureData = getEventPlotPoints("Aanval", beginDate, endDate);
		if (seizureData.length > 0) {
			chart.addData(seizureData,"Aanval","rgba(114, 191, 68, 0.8)");
		}
	}
	
	if (isSelected("Medicijngebruik")) {
		chart.addMedicines(beginDate, endDate);
	}
	
    if (isSelected("Bijwerking")) {
        var bijwerkingenData = getEventPlotPoints("Bijwerking", beginDate, endDate);
        if (bijwerkingenData.length > 0) {
            chart.addEventData("Bijwerking", bijwerkingenData);            
        }
    }

    for ( var eventName in EP_ALGEMEEN_TYPE) {
		if (isSelected(eventName)) {
			var eventData = getEventPlotPoints(eventName, beginDate, endDate);
			if (eventData.length > 0) {
				chart.addEventData('i18n_' + eventName, eventData);
			}
		}
	}

	if (isSelected("IncidenteleMedicatie")) {
		var incMedData = getEventPlotPoints("IncidenteleMedicatie", beginDate, endDate);
		if (incMedData.length > 0) {
			chart.addEventData("IncidenteleMedicatie", incMedData);
		}
	}

	chart.setXaxisRange(beginDate, endDate);	
	chart.drawIn('#placeholder');

	//
	// /*
	// * Inserting logic to save the graph to an image file
	// */
	//
	// if ($.support.cssFloat) { // currently evals to False in IE
	// $("#convertpngbtn").click(function() {
	// chart.saveAsPNG("placeholder", "report");
	// });
	// } else {
	// $("#convertpngbtn").click(function() {
	// alert("Image Exporting not available in IE");
	// });
	// }
};

$(function() {
	/*
	 * Redraw the chart with new orientation data
	 */
	$(window).resize(function() {
		if (chart != undefined) {
			chart.drawIn('#placeholder');
		}
		
		if ( $('#calendarWeekPage').hasClass('ui-page-active') ) {
			redrawCalendar();
		}
		if ( $('#calendarMonthPage').hasClass('ui-page-active') ) {
			redrawMonthCalendar();
		}
	});
});