var accountController = {

	account : {},

	saveAccount : function() {

		accountController.account.username = $("#accountUsername").val();
		accountController.account.email = $("#accountEmail").val();
		accountController.account.mobile = $("#accountMobile").val();
		accountController.account.pass1 = $("#accountPass1").val();
		accountController.account.pass2 = $("#accountPass2").val();
		accountController.account.country = $('#countryLang option:selected').val();

		// Alle velden valideren
		var allValid = true;
		var message = null;
		if (!validator.validateIsEmail(accountController.account.email)) {
			return false;
		}
		if (!validator.validateIsPhoneNumber(accountController.account.mobile)) {
			return false;
		}
		if (!validator.validateUsername(accountController.account.username)) {
			return false;
		}
		if (!validator.validatePassword(accountController.account.pass1)) {
			return false;
		}
		if (accountController.account.pass1 !== accountController.account.pass2) {
			phoneAlert(jQuery.i18n.prop('i18n_validation_error_passnotequal'));
			return false;
		}

		var jsonCreateAccount = {
			"gebruikersNaam" : accountController.account.username,
			"emailAdres" : accountController.account.email,
			"mobieleNummer" : accountController.account.mobile,
			"land" : accountController.account.country,
			"language" : getSettings().language,
			"wachtwoord" : hex_sha256( accountController.account.pass1 + "{" + EP_PASSWORD_SALT + "}")
		};

		serviceController.sendRequest(EP_ACCOUNT_ENDPOINT, EP_ACCOUNT_NEW, jsonCreateAccount,
				accountController.newSuccess, accountController.newError);
	},

	newSuccess : function(data, textStatus, jqXHR) {
		if(data && data.status === "ERROR") {
			phoneAlert($.i18n.prop('i18n_account_error')  + " - " + data.message);
		} else {
			navigationController.gotoPreviousPage();

			var acc = accountController.getOrCreateAccount();
			acc.username = accountController.account.username;
			acc.password = accountController.account.pass1;

			storeJSON(acc, acc.id);

			phoneAlert($.i18n.prop('i18n_account_success') );
		}
	},

	newError : function(jqXHR, textStatus, errorThrown) {
		phoneAlert($.i18n.prop('i18n_account_error'));
	},

	activateSuccess : function(response) {
		if(response.status == "SUCCESS") {
			phoneAlert($.i18n.prop('i18n_account_activate_success'));
		} else {
			phoneAlert($.i18n.prop('i18n_account_activate_error') + " - " + response.message);
		}
	},

	activateError : function() {
		phoneAlert($.i18n.prop('i18n_account_activate_error'));
	},

	activateAccount : function(username, activatieCode) {
		var jsonActivate = {
			"gebruikersNaam" : username,
			"activatieCode" : activatieCode
		};

		serviceController.sendRequest(EP_ACCOUNT_ENDPOINT, EP_ACCOUNT_ACTIVATE, jsonActivate,
				accountController.activateSuccess, accountController.activateError);
	},

	requestNewActivationCode : function( username ) {
		var jsonNewCode = {
				"gebruikersNaam" : username
		};
		
		serviceController.sendRequest(EP_ACCOUNT_ENDPOINT, EP_ACCOUNT_NEW_CODE, jsonNewCode,
				accountController.newcodeSuccess, accountController.newcodeError);
	},
	
	newcodeSuccess : function(response) {
		if(response.status == "SUCCESS") {
			phoneAlert($.i18n.prop('i18n_account_newcode_success'));
		} else {
			phoneAlert($.i18n.prop('i18n_account_newcode_error') + " - " + response.message);
		}
	},
	
	newcodeError : function() {
		phoneAlert($.i18n.prop('i18n_account_newcode_error'));
	},
	
	login : function(user, pass) {
		var jsonLogin = {
			"gebruikersNaam" : user,
			"wachtwoord" : hex_sha256( pass + "{" + EP_PASSWORD_SALT + "}")
		};

		var account = accountController.getOrCreateAccount();

		account.username = user;
		account.password = pass;

		storeJSON(account, account.id);

		serviceController.sendRequest(EP_ACCOUNT_ENDPOINT, EP_ACCOUNT_LOGIN, jsonLogin, accountController.loginSuccess,
				accountController.loginError);
	},

	loginSuccess : function(response) {
		if (response.status == "SUCCESS") {
			phoneAlert($.i18n.prop('i18n_account_login_success'));
			var account = accountController.getOrCreateAccount();
			account.loggedIn = true;

			storeJSON(account, account.id);

			updateLoginForm();
		} else {
			phoneAlert($.i18n.prop('i18n_account_login_error') + " - " + response.message);
		}
	},

	loginError : function(response) {
		phoneAlert($.i18n.prop('i18n_account_login_error'));
	},
	
	logout : function()  {
		var account = accountController.getOrCreateAccount();
		account.loggedIn = false;

		storeJSON(account, account.id);
		updateLoginForm();
	},

	getOrCreateAccount : function() {
		var accArr = getJSON("epAccount");
		var acc = null;

		if (accArr == null) {
			acc = new epAccount();
			storeJSON(acc);
		} else {
			acc = accArr[0];
		}

		return acc;
	}

};

/**
 * Login form moet readonly worden als gebruiker is ingelogd. De inlog knop moet
 * dan veranderen in een uitlog knop.
 */
function updateLoginForm() {
	var account = accountController.getOrCreateAccount();

	var loginField = $("#accountUsernameLogin");
	var passField = $("#accountPasswordLogin");
	var loginButton = $("#account_inloggen");
	var logoutButton = $("#account_uitloggen");

	if (account.loggedIn == true) {
		loginButton.hide();
		logoutButton.show();
		
		loginField.val(account.username);
		passField.val(account.password);

		loginField.attr('disabled', 'disabled');
		passField.attr('disabled', 'disabled');


	} else {
		loginButton.show();
		logoutButton.hide();
		
		loginField.removeAttr('disabled');
		passField.removeAttr('disabled');
	}
}

$(function() {

	$("#accountAanmaken").on('pagebeforeshow', function() {
		var countryList = $("#countryLang");

		// add option to select list
		countryList.html("");
		countryList.append($('<option value="BE">België</option>'));
		countryList.append($('<option value="GB">Great Brittain</option>'));
		countryList.append($('<option value="NL">Nederland</option>'));
		countryList.selectmenu('refresh', true);
	});
	
	// Account gegevens vast invullen als die beschikbaar zijn
	$("#accountPage").on('pagebeforeshow', function() {
		var account = accountController.getOrCreateAccount();
		updateLoginForm();
		
		if (account != null) {
			$("#accountUsernameLogin").val(account.username);
			$("#accountPasswordLogin").val(account.password);
		}
	});

	// Account activeren username vast vooraf invullen
	$("#accountActiveren").on('pagebeforeshow', function() {
		var account = accountController.getOrCreateAccount();

		if (account != null) {
			$("#accountActivateUsername").val(account.username);
		}
	});

	// Login
	$("#account_inloggen").on('click', function() {
		var login = $("#accountUsernameLogin").val();
		var pass = $("#accountPasswordLogin").val();

		accountController.login(login, pass);
	});
	
	// Logout
	$("#account_uitloggen").on('click', function() {
		accountController.logout();
	});

	// Als een account wordt aangemaakt, eerst valideren en daarna een server
	// call doen
	$("#accountCreateButton").on('click', function() {
		accountController.saveAccount();
	});

	// Account activeren
	$("#accountActivateButton").on('click', function() {
		var user = $("#accountActivateUsername").val();
		var code = $("#accountCode").val();

		accountController.activateAccount(user, code);
	});
	
	// Nieuwe activatie code aanvragen
	$("#accountRequestNew").on('click', function() {
		var user = $("#accountActivateUsername").val();
		
		accountController.requestNewActivationCode(user);
	});

});
