/**
 * Controller voor Afwijkende Medicijn Gebruik gebeurtenissen.
 */
var afwijkendeMedicatieController = epExtend(eventController, {

	/**
	 * Override. Deze functie wordt aangeroepen wanneer er een nieuw medicijn
	 * moet worden aangemaakt.Form initializeren. Leegmaken van bepaalde velden
	 * en datum/tijd prefillen.
	 */
	New : function(medicine, daypart) {
		this._super.New();
		navigationController.gotoPage("#editAfwijkendMedicijngebruik");
		var currentDateTime = this._super.getNewEventDate();

		var events = afwijkendeMedicatieController.convertToCustomMedicineEvents(medicine, currentDateTime);
		var event = null;
		for (e in events) {
			if (events[e].daypart == daypart) {
				event = events[e];
			}
		}

		if (event != null) {
			// var dateTime = Date.parseExact(event.startDateTime, "yyyy/MM/dd
			// H:mm");
			this.FillHtmlElements(event, event.startDateTime);
		}
	},

	NewForBehandelSchema : function(medicineschema, daypart_time) {
		this._super.New();
		navigationController.gotoPage("#editAfwijkendMedicijngebruik");
		var currentDateTime = this._super.getNewEventDate();

		var events = afwijkendeMedicatieController.convertBehandelSchemaToCustomMedicineEvents([ medicineschema ], currentDateTime);
		var event = null;
		for (e in events) {
			if (events[e].time == daypart_time) {
				event = events[e];
			}
		}

		if (event != null) {
			// var dateTime = Date.parseExact(event.startDateTime, "yyyy/MM/dd
			// HH:mm");
			this.FillHtmlElements(event, event.startDateTime);
		}
	},

	FillHtmlElements : function(event, dateTime) {
		if (event.behandelschemaServerId) {
			$('#afwijkendServerId').html(event.behandelschemaServerId);
		}
		$('#afwijkendMedicijnNaam').html(event.drugName);
		$("#afwijkendMedicijnDatum").html(dateTime.toDisplayDateString());
		$("#afwijkendMedicijnTijd").html(dateTime.toDisplayTimeString());
		$("#afwijkendMedicijnAantal").val(event.quantity);
		$("#afwijkendMedicijnUnit").val(event.unit);
		$('#afwijkendMedicijnUnit').selectmenu('refresh');
		$('#afwijkendMedicijnMissedOrExtra').val(event.reason).selectmenu("refresh");
	},

	/**
	 * Deze functie wordt aangeroepen als een bestaand event wordt bewerkt.
	 * 
	 * @param event:
	 *           Het event object dat bewerkt moet worden.
	 */
	Edit : function(event) {
		this._super.Edit(event);

		navigationController.gotoPage("#editAfwijkendMedicijngebruik");
		var dateTime = Date.parse(event.startDateTime);

		// vul form
		if (event.behandelschemaServerId) {
			$('#afwijkendServerId').html(event.behandelschemaServerId);
		}
		$("#afwijkendMedicijnNaam").html(event.drugName);
		$("#afwijkendMedicijnDatum").html(dateTime.toDisplayDateString());
		$("#afwijkendMedicijnTijd").html(dateTime.toDisplayTimeString());
		$("#afwijkendMedicijnAantal").val(event.quantity);
		$("#afwijkendMedicijnUnit").val(event.unit);
		$('#afwijkendMedicijnUnit').selectmenu('refresh');
		$('#afwijkendMedicijnMissedOrExtra').val(event.reason).selectmenu("refresh");
	},

	/**
	 * Converteert een epRecurringMedicine object naar een array van afwijkende
	 * medicijn objecten. We geven de resulterende events allemaal hetzelfde ID
	 * 
	 * @param medicine
	 */
	convertToCustomMedicineEvents : function(medicine, date) {
		var events = [];
		for (i in medicine.dayparts) {
			var dp = medicine.dayparts[i];
			if (dp.time.length != 5) {
				dp.time = '0' + dp.time; // prepend zero if necessary
			}
			var datestring = date.toString('yyyy-MM-ddT') + dp.time;
			var storageDate = Date.parseExact(datestring, 'yyyy-MM-ddTHH:mm');
			var event = new epEventCustomMedicine(storageDate, medicine);
			event.id = medicine.id;
			event.time = dp.time;
			event.quantity = dp.quantity;
			event.daypart = dp.daypart;
			event.unit = medicine.unit;
			event.eventType.name = "Medicijngebruik";
			events.push(event);
		}
		return events;
	},

	/**
	 * Conversie van behandelschema dagen naar custommedicine events om ze in de
	 * kalender te kunnen tonen. We gaan er hierbij vanuit dat van elke dag
	 * altijd de 4 momenten gesynct worden, dat scheelt enorm in het ophalen van
	 * de bepaalde dag, omdat we dan niet de hele lijst door hoeven, maar op
	 * index kunnen selecteren. Wel opletten dus als het aantal momenten per dag
	 * gaat toenemen (dat scenario leek niet waarschijjnlijk omdat dit voor
	 * andere apps ook vast staat op 4 per dag)
	 * 
	 * @param schemas een of meerdere medicijnschemas
	 * @date datum waarvoor de events moeten worden gegenereerd
	 */
	convertBehandelSchemaToCustomMedicineEvents : function(schemas, date) {
		var events = [];

		for (i in schemas) {
			var schema = schemas[i];
			var startdate = Date.parseExact(schema.startdate, 'yyyy-MM-ddTHH:mm:ss').clearTime();
			var diffInDays = getDifferenceInDays(startdate, date.clearTime());
			var startIndex = diffInDays * 4;
			var endIndex = startIndex + 4;
			if (diffInDays >= 0 && endIndex <= schema.innameMomenten.length ) {
				for ( var j = startIndex; j < endIndex; j++) {
					if (schema.innameMomenten[j].quantity > 0) {
						var storageDate = Date.parseExact(schema.innameMomenten[j].date, 'yyyy-MM-ddTHH:mm:ss');
						var event = new epEventCustomMedicine(storageDate);
						event.drugName = schema.drugName;
						event.id = schema.serverId;
						event.time = storageDate.toString("HH:mm");
						event.quantity = schema.innameMomenten[j].quantity;
						event.behandelschemaServerId = schema.serverId;
						event.unit = schema.unit;
						event.eventType.name = "Medicijngebruik";
						events.push(event);
					}
				}				
			}
		}
		return events;
	}

});

/**
 * Viewer gedeelte. Afhandelen en registreren van GUI events.
 */
$(function() {

	$('#editAfwijkendMedicijngebruikLink').click(function() {
		navigationController.gotoPage("#editAfwijkendMedicijngebruik", true);
		medicijnController.New();
	});

	function mergeDateTime(date, time) {
		var dateTimeArray = [ date, " ", time ];
		return Date.parse(dateTimeArray.join(''));
	}
	/*
	 * Incidentele medicatie form submission afhandelen
	 */
	$('#afwijkendMedicijnSubmit').on("click", function() {
		var message = "";

		var serverId = $("#afwijkendServerId").html();
		var drugDate = $("#afwijkendMedicijnDatum").html();
		var drugTime = $("#afwijkendMedicijnTijd").html();

		var drugName = $("#afwijkendMedicijnNaam").html();
		var dateTime = mergeDateTime(drugDate, drugTime);

		var newDrug = new epEventCustomMedicine(dateTime);

		// Retrieve all filled in fields
		if (serverId && serverId != "") {
			newDrug.behandelschemaServerId = serverId;
		}
		newDrug.drugName = drugName;
		newDrug.unit = $("#afwijkendMedicijnUnit").val();
		newDrug.quantity = $("#afwijkendMedicijnAantal").val();
		newDrug.reason = $("#afwijkendMedicijnMissedOrExtra").val();
		newDrug.eindDateTime = dateTime;
		newDrug.time = drugTime;

		// numerieke validatie is toch nodig omdat je zelfs met het telefoon
		// keyboard kan pasten...
		if (!validator.validateStringIsFloatNumberGreaterThanZero(newDrug.quantity)) {
			return false;
		}

		if (eventController.getAction() == ACTION.Edit) {
			eventController.update(newDrug);
		} else {
			eventController.save(newDrug);
		}
		eventController.Clear();

		navigationController.gotoPreviousPage();
		return false;
	});
});
