/**
 * Viewer en Controller voor 'Algemene gebeurtenissen' zoals bv alcohol gebruik,
 * slaap ritme.
 */
var algemeneGebeurtenisController = epExtend(eventController, {

	/**
	 * Override. Nieuwe gebeurtenis aanmaken
	 */
	New : function() {
		this._super.New();
		var currentDateTime = this._super.getNewEventDate();
		
		

		// Clear form en startdatum/tijd op de huidige zetten.
		$("#editCustomNote").val('');
		$("#editCustomDatum").val(currentDateTime.toDisplayDateString());
		$("#editCustomDatumEnd").val(currentDateTime.toDisplayDateString());
		$("#editCustomStartTijd").val(currentDateTime.toDisplayTimeString());
		$("#editCustomEindTijd").val(currentDateTime.addHours(1).toDisplayTimeString());
	},

	/**
	 * Override. Bestaande algemene gebeurtenis bewerken.
	 * 
	 * @param event
	 */
	Edit : function(event) {
		this._super.Edit(event);

		 navigationController.gotoPage("#editCustom");
		var dateTimeStart = Date.parse(event.startDateTime);
		var dateTimeEind = Date.parse(event.eindDateTime);

		// afkijken bij aanval controller
		$("#editCustomDatum").val(dateTimeStart.toDisplayDateString());
		$("#editCustomDatumEnd").val(dateTimeEind.toDisplayDateString());
		$("#editCustomStartTijd").val(dateTimeStart.toDisplayTimeString());
		$("#editCustomEindTijd").val(dateTimeEind.toDisplayTimeString());

		// Event naam kan niet worden gewijzigd tijdens bewerken
		$("#editCustomNameGroup").hide();
		$("#editCustomHeaderImage").attr('src', "img/" + event.eventType.name + ".png");
		$("#editCustomHeader").html(jQuery.i18n.prop("i18n_" + event.eventType.name));

		$("#editCustomEigenGebeurtenis").attr("value", event.eventType.name);
		$("#editCustomNote").val(event.notes);
	}

});

$(function () {
    /* Dynamisch aanvullen van gebeurtenissenlijst */
    $('#eventDetailsPage').on("pageinit", function () {
        createAlgemeneGebeurtenissen();
    });

    /*
    * Eigen bijzonderheden form submission afhandelen
    */
    $('#editCustomSubmit').on("click", function () {
        var editEigenGebeurtenis = $("#editCustomEigenGebeurtenis").attr("value");

        /*
        * Omdat we hetzelfde form gebruiken voor 'Eigen gebeurtenis' als voor
        * 'Algemene gebeurtenis' checken we op de waarde van het hidden field.
        */
        if (editEigenGebeurtenis != "eigenGebeurtenis") {
            var eventDate = $("#editCustomDatum").val();
            var eventDateEnd = $("#editCustomDatumEnd").val();
            var eventStartTime = $("#editCustomStartTijd").val();
            var eventEindTime = $("#editCustomEindTijd").val();
            var eventNotes = $("#editCustomNote").val();
            
  
            var dateTimeStart = Date.parse(eventDate + " " + eventStartTime);
            var dateTimeEind = Date.parse(eventDateEnd + " " + eventEindTime);

            if (!validator.validateStarttimePreceedsEndTime(dateTimeStart, dateTimeEind)) {
                return false;
            }

            /* ALGEMENE GEBEURTENIS OPSLAAN */
            var gebeurtenisType = $("#editCustomEigenGebeurtenis").attr("value");
            var algemeenEvent = new epEventAlgemeen(dateTimeStart, gebeurtenisType);
            algemeenEvent.notes = eventNotes;
            algemeenEvent.eindDateTime = dateTimeEind;

            if (eventController.getAction() == ACTION.Edit) {
                eventController.update(algemeenEvent);
            } else {
                eventController.save(algemeenEvent);
            }
            eventController.Clear();

            navigationController.gotoPreviousPage();
        }

        return false;
    });

});

/*
 * Vul de lijst met algemene gebeurtenissen aan de hand van de enum
 * EP_ALGEMEEN_TYPE
 */
function createAlgemeneGebeurtenissen() {
	for (gebeurtenis in EP_ALGEMEEN_TYPE) {
		var img = '<img src="img/' + gebeurtenis + '.png" class="event-icon" />';
		var h3 = '<h3 class="i18n_' + gebeurtenis + '">' + jQuery.i18n.prop("i18n_" + gebeurtenis) + "</h3";
		$("#eventOverview").append('<li class="clickable editAlgemeenLink">' + img + h3 + "</li>");
	}
	$("#eventOverview").listview("refresh");

	/* Voeg een click event toe aan elk menu item */
	$(".editAlgemeenLink").click(function() {
		navigationController.gotoPage("#editCustom", true);

		algemeneGebeurtenisController.New();

		// Titel vooraf invullen als header, titelveld hiden
		var className = $(this).children("h3").attr("class").match(/i18n[^\s]+/g)[0];
		$("#editCustomNameGroup").hide();
		$("#editCustomHeaderImage").attr('src', "img/" + className.substr(5) + ".png");
		$("#editCustomHeader").html(jQuery.i18n.prop(className));

		/*
		 * Geef het hidden field de naam van de algemene gebeurtenis, zonder
		 * i18n_ ervoor)
		 */
		$("#editCustomEigenGebeurtenis").attr("value", className.toString().substr(5));
	});
}
