var behandelschemaController = epExtend(controller, {

	convertEenheid : function(eenheid) {
		if (eenheid == "stuks") {
			return 0;
		} else if (eenheid == "gram") {
			return 1;
		} else if (eenheid == "milliliter") {
			return 2;
		} else if (eenheid == "milligram") {
			return 3;
		}
	},
	
	get : function() {
		var schemaArray = getJSON("epArtsBehandelSchema");
		return schemaArray ? schemaArray[0] : null;
	},
	
	getBehandelschemaFromServer : function() {
		serviceController.getRequest(EP_SYNCHRONISE_ENDPOINT, EP_GET_ARTSSCHEMA, behandelschemaController.onGetBehandelschemaSuccess, behandelschemaController.onGetBehandelSchemaFailure);
	},

	onGetBehandelSchemaFailure : function() {
		phoneAlert(jQuery.i18n.prop('i18n_behandelschema_get_failure'));
	},
	
	onGetBehandelschemaSuccess : function(JSONschema) {
		if (JSONschema.status == "SUCCESS") {
			navigationController.gotoPage("#behandelschemaInitPage");
			var medicijnen = [];
			for (i in JSONschema.medicijnSchemas) {
				medicijnen.push(JSONschema.medicijnSchemas[i].medicijnNaam);
			}
			$("#behandelschemaDescription").html(medicijnen.join("\n"));
			
			sessionStorageWrapper().SetControllerObject(JSON.stringify(JSONschema));
		} else {
			phoneAlert(jQuery.i18n.prop('i18n_behandelschema_get_failure_no_scheme'));
		}
	},

	confirmReplaceBehandelschema : function() {
		var startdate_string = $('#behandelschemaStartDate').val();
		var startdate = Date.parse(startdate_string);
		if (startdate != null) {
			var jsonschema_string = sessionStorageWrapper().GetControllerObject();
			var JSONschema = JSON.parse(jsonschema_string);
			behandelschemaController.processBehandelschema(JSONschema, startdate);		
			sessionStorageWrapper().RemoveControllerObject();
			navigationController.gotoPage("#mijnMedicijnenPage");
		}
	},
	
	processBehandelschema : function(json, startdate) {

		var bs = new epArtsBehandelSchema();
		for (i in json.medicijnSchemas) {
			jschema = json.medicijnSchemas[i];
			var ms = new epBehandelSchema(jschema.medicijnNaam, jschema.serverId);
			ms.unit = behandelschemaController.convertEenheid(jschema.eenheid);
			ms.startdate = startdate;
			
			for (var im in jschema.innameMomenten) {
				var jdag = jschema.innameMomenten[im];
				var datestring = startdate.toString('yyyy/MM/dd ') + jdag.innameTijdstip;
				var innameDag = Date.parseExact(datestring, 'yyyy/MM/dd HH:mm');
				innameDag.addDays(jdag.innameDag - 1);	
				var innameMoment = new epInnameMoment(innameDag, jdag.hoeveelheid);
				ms.innameMomenten.push(innameMoment);
			}	
			
			bs.medicijnSchemas.push(ms);
		}
		
		var existingSchema = behandelschemaController.get();
		if (existingSchema && existingSchema != null) {
			removeJSON('epArtsBehandelSchema', existingSchema.id);
		}
		
		storeJSON(bs);
	},
	
	Edit : function(medicijnId) {
		// fill form with recurring event
		var behandelschema = behandelschemaController.get();
		var medicijnschema = behandelschemaController.getMedicijnSchema(medicijnId);
		this._super.Edit(medicijnschema);
		
		$("#behandelschemaMedicineName").html(medicijnschema.drugName);
		var dateString = Date.parse(behandelschema.dateCreated).toDateString();
		$("#downloadDate").html(dateString);
		$("#notifyBehandelschemaSwitch").prop("checked", medicijnschema.sendNotification);
		$("#notifyBehandelschemaSwitch").checkboxradio('refresh');
	},
	
	getMedicijnSchema : function(medicijnId) {
		var artsSchema = behandelschemaController.get();
		for (var i in artsSchema.medicijnSchemas) {
			var id = artsSchema.medicijnSchemas[i].id;
			if (id == medicijnId) {
				return artsSchema.medicijnSchemas[i];
			}
		}
		
		return null;
	},

	getMedicijnSchemaByServerId : function(serverId) {
		var artsSchema = behandelschemaController.get();
		if (artsSchema && artsSchema != null) {
			for (var i in artsSchema.medicijnSchemas) {
				var id = artsSchema.medicijnSchemas[i].serverId;
				if (id == serverId) {
					return artsSchema.medicijnSchemas[i];
				}
			}			
		}
		
		return null;
	},
	
	deleteMedicijnSchema : function(medicijnId) {
		var artsSchema = behandelschemaController.get();
		for (var i in artsSchema.medicijnSchemas) {
			var id = artsSchema.medicijnSchemas[i].id;
			if (id == medicijnId) {
				artsSchema.medicijnSchemas.splice(i, 1);
				storeJSON(artsSchema, artsSchema.id);
				break;
			}
		}
	},

	/**
	 * Update van een medicijnschema. Nodig om user-specifieke settings op te kunnen slaan, bv. notifications.
	 * @param medicijnschema
	 */
	save : function(medicijnschema) {
		var behandelschema = behandelschemaController.get();
		for (var i in behandelschema.medicijnSchemas) {
			var schema = behandelschema.medicijnSchemas[i];
			if (schema.serverId == medicijnschema.serverId) {
				behandelschema.medicijnSchemas[i] = medicijnschema;
				storeJSON(behandelschema, behandelschema.id);
				break;
			}
		}
	},
	
	setNotification : function(medicijnschema) {

		var base_message = jQuery.i18n.prop('i18n_medicine_notification_message');
		
		// enable
		for (i in medicijnschema.innameMomenten) {
			var moment = medicijnschema.innameMomenten[i];
			var notificationId = medicijnschema.id + '-' + i;
			var medicineUnit = jQuery.i18n.prop(getEnumByValue(EP_EVENT_INCIDENT_DRUG_UNIT, medicijnschema.unit));
			// enabled
			if (medicijnschema.sendNotification) {
				var message = base_message + ": \r\n" + moment.quantity + " " + medicineUnit + " - " + medicijnschema.drugName;
				var date = Date.parseExact(moment.date, 'yyyy-MM-ddTHH:mm:ss');
				NotificationService.addSingleNotification(notificationId, date, message);
			} else {
				NotificationService.removeNotification(notificationId);
			}
		}

	},
	
	removeNotification : function(medicijnschema) {
		if (medicijnschema.sendNotification) {
			for (i in medicijnschema.innameMomenten) {
				var notificationId = medicine.id + '-' + i;
				NotificationService.removeNotification(notificationId);
			}
		}
	}
	
});

$(function() {
	/*
	 * edit link voor het bewerken van een eigen medicijn. Het id an de link
	 * verwijst naar het item in storage.
	 */
	$("#saveBehandelschemaLink").live("click", function() {
		var medicijnSchemaId = behandelschemaController.getObjectId();
		var medicijnschema = behandelschemaController.getMedicijnSchema(medicijnSchemaId);
			
		medicijnschema.sendNotification = $("#notifyBehandelschemaSwitch").is(":checked");
		behandelschemaController.setNotification(medicijnschema);
		behandelschemaController.save(medicijnschema);
		
		navigationController.gotoPage("#mijnMedicijnenPage");
	});

	$("#behandelschemaDetailsLink").live("click", function() {
		navigationController.gotoPage("#medicatieDetailsPage");
		var medicijnSchemaId = behandelschemaController.getObjectId();
		
		if (medicijnSchemaId && medicijnSchemaId != null) {
			mijnMedicatieDetailsController.ViewBehandelschema(medicijnSchemaId);
		}
	});

	$("#behandelschemaDeleteLink").live("click", function() {
		var medicijnSchemaId = behandelschemaController.getObjectId();
		behandelschemaController.deleteMedicijnSchema(medicijnSchemaId);
		behandelschemaController.removeNotification(medicijnSchemaId);
		
		navigationController.gotoPage("#mijnMedicijnenPage");
	});

	$("#getBehandelschemaButton").live("click", function() {
		behandelschemaController.getBehandelschemaFromServer();		
	});

	$("#behandelschemaStartButton").live("click", function() {
		phoneConfirm(jQuery.i18n.prop('i18n_behandelschema_are_you_sure_replace_scheme'), behandelschemaController.confirmReplaceBehandelschema, null, jQuery.i18n.prop('i18n_behandelschema_replace_scheme'));
	});

	
});