/**
* Controller voor Incidenteel Medicijn Gebruik gebeurtenissen.
*/
var bijwerkingenController = epExtend(eventController, {

    /**
    * Override. Deze functie wordt aangeroepen wanneer er een nieuw medicijn
    * moet worden aangemaakt.Form initializeren. Leegmaken van bepaalde velden
    * en datum/tijd prefillen.
    */
    New: function () {
        this._super.New();
        var currentDateTime = this._super.getNewEventDate();

        $('#sideEffectName').add("#sideEffectNote").not(':submit').val('');
        $("#sideEffectStartDate").val(currentDateTime.toDisplayDateString());
        $("#sideEffectStartTime").val(currentDateTime.toDisplayTimeString());
        $("#sideEffectEndDate").val(currentDateTime.toDisplayDateString());
        $("#sideEffectEndTime").val(currentDateTime.addHours(1).toDisplayTimeString());
        $("#sideEffectIntensity").val('1');
    },

    /**
    * Deze functie wordt aangeroepen als een bestaand event wordt bewerkt.
    * 
    * @param event:
    *            Het event object dat bewerkt moet worden.
    */
    Edit: function (event) {
        this._super.Edit(event);

        navigationController.gotoPage("#editBijwerking");
        var startDateTime = Date.parse(event.startDateTime);
        var endDateTime = Date.parse(event.eindDateTime);

        // vul form
        $('#sideEffectName').val(event.name);
        $("#sideEffectStartDate").val(startDateTime.toDisplayDateString());
        $("#sideEffectStartTime").val(startDateTime.toDisplayTimeString());
        $("#sideEffectEndDate").val(endDateTime.toDisplayDateString());
        $("#sideEffectEndTime").val(endDateTime.toDisplayTimeString());


        $("#sideEffectIntensity").val(event.intensity);
        $("#sideEffectNote").val(event.notes);
    }

});

/**
* Viewer gedeelte. Afhandelen en registreren van GUI events.
*/
$(function () {

    $('#editBijwerking').on('pageinit', function () {
        // koppelen van button aan drugname veld.
        createIconLink($('#sideEffectName'), $('#sideEffectLinkList'), function () {
        	navigationController.gotoPage("#kiesBijwerking");
        });
        createBijwerkingenList("#sideEffectName");
    });

    $('#editBijwerkingLink').click(function () {
    	navigationController.gotoPage("#editBijwerking", true);
        bijwerkingenController.New();
    });

    /*
    * Incidentele medicatie form submission afhandelen
    */
    $('#sideEffectSubmit').on("click", function () {
        var message = "";
        var name = $("#sideEffectName").val();
        var startDate = $("#sideEffectStartDate").val();
        var startTime = $("#sideEffectStartTime").val();
        var endDate = $("#sideEffectEndDate").val();
        var endTime = $("#sideEffectEndTime").val();
        var intensity = $("#sideEffectIntensity").val();
        var notes = $("#sideEffectNote").val();

        var startDateTime = Date.parse(startDate + " " + startTime);

        var newSideEffect = new epEventSideEffect(startDateTime, name);

        // Retrieve all filled in fields
        newSideEffect.eindDateTime = Date.parse(endDate + " " + endTime);
        newSideEffect.intensity = intensity;
        newSideEffect.notes = notes;


        // validate input
        if (newSideEffect.name === "") {
            message = $.i18n.prop("i18n_validation_error_enter_name_for_sideeffect");
            phoneAlert(message);
            return false;
        }
        if (!validator.validateStarttimePreceedsEndTime(newSideEffect.startDateTime, newSideEffect.eindDateTime)) {
            return false;
        }


        if (eventController.getAction() == ACTION.Edit) {
            eventController.update(newSideEffect);
        } else {
            eventController.save(newSideEffect);
        }
        eventController.Clear();

        navigationController.gotoPreviousPage();	

        return false;
    });
});

/**
* @param targetInputId
*            de id van de inputbox waaraan de quicklistbutton gekoppeld is
*/
function createBijwerkingenList(targetInputId) {
    // vind de parent van de inputbox, omdat we terug willen navigeren.
    var parentPageId = $(targetInputId).closest('div[data-role="page"]').attr("id");
    /*
    * Als er een medicijn uit de snelkeuze lijst is geselecteerd, vul de waarde
    * dan in op de vorige pagina en navigeer daar naar terug.
    */
    $("#sideEffectList > li").unbind('click'); //

    /*
    * nodig omdat anders de oude clickevents blijven (kan een andere pagina
    * zijn)
    */
    $("#sideEffectList > li").on('click', function () {
    	navigationController.gotoPage("#" + parentPageId, true);
        $(targetInputId).val($("p", this).text());
    });
}
