var contactenController = epExtend(controller, {

	/**
	 * Override. Nieuwe contactpersoon toevoegen.
	 */
	New : function() {
		this.Clear();
		this._super.New();
		// clear session and set to new

		// clear/init form
		$("#contactName").val('');
		$("#contactPhonenumber").val('');
		$("#contactEmail").val('');
		$("#contactRelation").val('');
		$("#contactIsQuickDial").attr('checked', false).checkboxradio("refresh");
		$("#contactIsFollower").attr('checked', false).checkboxradio("refresh");
	},

	/**
	 * Override. Bestaande contactpersoon bewerken
	 * 
	 * @param contact
	 */
	Edit : function(contact) {
		this._super.Edit(contact);

		navigationController.gotoPage("#editContactPage");

		// fill form with recurring event
		$("#contactName").val(contact.name);
		$("#contactPhonenumber").val(contact.phoneNumber);
		$("#contactEmail").val(contact.email);
		$("#contactRelation").val(contact.role);
		$("#contactIsQuickDial").attr('checked', contact.isQuickDial).checkboxradio("refresh");
		$("#contactIsFollower").attr('checked', contact.isFollower).checkboxradio("refresh");
	},

	/**
	 * Override. Maar geen call naar super omdat deze update andere
	 * functionaliteit moet hebben.
	 * 
	 * @param contact
	 */
	update : function(contact) {
		/*
		 * We must use the ID from the controller and not the ID of the passed
		 * event, since that ID contains a newly generated ID. So this ID won't
		 * match the existing stored object id
		 */
		contact.id = this.getObjectId();
		storeJSON(contact, contact.id);
	},

	listItemClick : function(event) {
		navigationController.gotoPage("#editContactPage");
		var storedContact = contactenController.getContact($(this).attr('id'));
		contactenController.Edit(storedContact);
	},

	buildContactList : function(contacts) {
		$('#lvContacts').empty();
		for ( var c in contacts) {
			var contact = contacts[c];
			var $template = $('.contact-template').clone().removeClass('contact-template');
			$template.addClass('editContactLink contactListItem').attr('id', contact.id);
			$template.find('.dayPartIcon').attr('src', 'img/user.png');
			var phoneIcon = contact.isQuickDial ? 'img/dial-icon.png' : "";
			var gpsIcon = contact.isFollower ? 'img/gps-icon.png' : "";
			$template.find('#phoneIcon').attr('src', phoneIcon);
			$template.find('#gpsIcon').attr('src', gpsIcon);
			$template.find('.dayPartDescription').html(contact.name);
			$template.find('.contactRelationDescription').html(contact.role);
			$('#lvContacts').append($template.show());
		}
		$('#lvContacts li').click(this.listItemClick);
		$('#lvContacts').listview('refresh');
	},

	createContacten : function() {
		this.buildContactList(this.getAll());
	},

	getAll : function() {
		return getJSON('epContact');
	},

	getContact : function(id) {
		return getJSON('epContact', id);
	},

	remove : function(id) {
		removeJSON('epContact', id);
	},

	/**
	 * Geeft terug of het maximale aantal contacten op de alarmpagina is
	 * overschreden. Deze functie wordt aangeroepen bij het editen van een
	 * contact, en dus mag je deze niet meetellen (want anders kan je het derde
	 * vinkje niet meer aan en uit toggelen).
	 * 
	 * @param thiscontact
	 * @returns {Boolean}
	 */
	numberOfQuickdialsExceeded : function(thiscontact) {
		var contacts = this.getAll();
		var total = 0;
		for ( var c in contacts) {
			if (contacts[c].isQuickDial && contacts[c].id != this.getObjectId()) {
				total++;
			}
		}

		return total >= EP_QUICKDIALS_MAX;
	}

});

$(function() {

	/*
	 * Opslaan van de contactpersoon
	 */
	$('#editContactSubmit').live("click", function() {
		var message = "";
		var contact = new epContact();
		contact.name = $("#contactName").val();
		contact.phoneNumber = $("#contactPhonenumber").val();
		contact.email = $("#contactEmail").val();
		contact.role = $("#contactRelation").val();
		contact.isQuickDial = $("#contactIsQuickDial").is(':checked');
		contact.isFollower = $("#contactIsFollower").is(':checked');

		if (contact.name === "") {
			message = $.i18n.prop("i18n_validation_error_name_for_contact");
			phoneAlert(message);
			return false;
		}


		if (!validator.validateIsPhoneNumber(contact.phoneNumber)) {
			return false;
		}
		
		if(!validator.validateIsEmail(contact.email)) {
			return false;
		}

		if (contactenController.getAction() == ACTION.Edit) {
			contactenController.update(contact);
		} else {
			contactenController.save(contact);
		}
		contactenController.Clear();

		navigationController.gotoPage("#contactsPage", true);

		return false;
	});

	/*
	 * Als de gebruiker een vinkje probeert te zetten bij de quickdial optie, en
	 * er zijn al EP_QUICKDIAL_MAX quickdials, kan deze niet worden gezet. De
	 * gebruiker moet dan eerst een andere uitzetten.
	 */
	$('#contactIsQuickDial').change(function() {

		if ($(this).is(':checked') && contactenController.numberOfQuickdialsExceeded()) {
			$(this).attr('checked', false).checkboxradio("refresh");
			phoneAlert(jQuery.i18n.prop('i18n_quickdials_exceeded'));
		}

	});

	$('#contactDeleteLink').live('click', function() {
		phoneConfirm(jQuery.i18n.prop('i18n_confirm_delete_contact'), function() {

			var contact = contactenController.getObjectId();

			contactenController.remove(contact);
			contactenController.Clear();

			navigationController.gotoPage("#contactsPage", true);
		});
	});

	$('#newContactLink').click(function() {
		navigationController.gotoPage("#editContactPage");
		contactenController.New();
	});

	$('#contactsPage').on("pagebeforeshow", function() {
		contactenController.createContacten();
	});
});
