/**
 * Global variable om de controller toestand mee te bepalen.
 */
var ACTION = new Enum([ "New", "Edit", "Clear" ]);

/**
 * Base class voor alle controllers in dit project.
 * 
 * Bevat basis functionaliteit voor een pagina om bij te houden of een form in
 * 'new' of 'edit' mode is.
 */
var controller = {

    callbackFunctions: [],

    /**
    * Zet de controller in 'New' modus.
    */
    New: function () {
        sessionStorageWrapper().SetControllerAction(ACTION.New);
    },

    /**
    * Zet de controller in 'Edit' modus. Sla tevens het ID van de epBase
    * (instance) op in de session storage.
    * 
    * @param obj
    *            Het epBase (instance) object wat bewerkt wordt.
    */
    Edit: function (obj) {
        sessionStorageWrapper().SetControllerAction(ACTION.Edit);
        sessionStorageWrapper().SetControllerObject(obj.id);
        sessionStorageWrapper().SetControllerObjectType(obj.objectType);
    },

    /**
    * Verwijder het object wat momenteel wordt ge-edit.
    * 
    * @returns {Boolean} true wanneer succesvol. Anders false.
    */
    Delete: function () {
        var objId = sessionStorageWrapper().GetControllerObject();
        var objType = sessionStorageWrapper().GetControllerObjectType();

        if (objId == null)
            return false;
        else {
            removeJSON(objType, objId);
            return true;
        }
    },

    /**
    * Aanroepen nadat 'New' / 'Edit' mode niet meer van toepassing is.
    * Opschonen van de tijdelijke boekhouding in de session storage.
    */
    Clear: function () {
        sessionStorageWrapper().RemoveControllerAction(); // to make sure
        sessionStorageWrapper().RemoveControllerObject(); // to make sure
        sessionStorageWrapper().RemoveControllerObjectType(); // to make sure
    },

    /**
    * Bijwerken van een bestaand epBase (instance) object.
    * 
    * @param obj
    *            Het bestaande epBase (instance) object dat moet worden
    *            bijgewerkt in de storage.
    */
    update: function (obj) {
        /**
        * We moeten het ID gebruiken wat in de controller is opgeslagen en niet
        * van het meegegeven object, deze kan namelijk een nieuwe gegenereerd
        * ID bevatten. Als we dat opslaan wordt er een extra object gemaakt.
        * Als dit de bedoeling is, gebruik dan controller.save();
        */
        obj.id = this.getObjectId();
        storeJSON(obj, obj.id);
    },

    /**
    * Opvragen van de modus waarin deze controller is.
    * 
    * @returns De huidige controller modus. Null wanneer deze niet is
    *          ingesteld, of is gecleared.
    * 
    * @see controller.ACTION variable.
    * @see this.Clear();
    */
    getAction: function () {
        var action = sessionStorageWrapper().GetControllerAction();
        return (action != undefined) ? action : null;
    },

    /**
    * Opvragen van het ID van het epBase (instance) object.
    * 
    * @returns Het id van het epBase object wat wordt bewerkt door deze
    *          controller. Of null wanneer controller niet in Edit modus is.
    */
    getObjectId: function () {
        return sessionStorageWrapper().GetControllerObject();
    },

    /**
    * Opslaan van het epBase (instance) object.
    * 
    * @param obj
    *            Het epBase (instance) object wat moet worden opgeslagen.
    */
    save: function (obj) {
        storeJSON(obj);
    }
};