/**
 * Viewer en Controller voor 'Eigen gebeurtenissen'.
 */
var eigenGebeurtenisController = epExtend(eventController, {

	/**
	 * @Override
	 * 
	 * Aanmaken van een nieuwe eigen gebeurtenis.
	 */
	New : function() {
		this._super.New();
		this.Init();

		var currentDateTime = this._super.getNewEventDate();

		$("#editCustomName").add("#editCustomNote").val('');

		// Clear form en startdatum/tijd op de huidige zetten.
		$("#editCustomNote").val('');
		$("#editCustomDatum").val(currentDateTime.toDisplayDateString());
		$("#editCustomDatumEnd").val(currentDateTime.toDisplayDateString());
		$("#editCustomStartTijd").val(currentDateTime.toDisplayTimeString());
		$("#editCustomEindTijd").val(currentDateTime.addHours(1).toDisplayTimeString());
	},

	/**
	 * @Override Bewerken van een bestaande 'Eigen gebeurtenis'
	 */
	Edit : function(event) {
		this._super.Edit(event);
		var dateTime = Date.parse(event.startDateTime);
		var dateTimeEnd = Date.parse(event.eindDateTime);

		navigationController.gotoPage("#editCustom");
		this.Init();

		// vul form
		$("#editCustomDatum").val(dateTime.toDisplayDateString());
		$("#editCustomDatumEnd").val(dateTimeEnd.toDisplayDateString());
		$("#editCustomStartTijd").val(dateTime.toDisplayTimeString());
		$("#editCustomEindTijd").val(dateTimeEnd.toDisplayTimeString());
		$("#editCustomName").val(event.eventType.eigenTitel);
		$("#editCustomNote").val(event.notes);
	},

	Init : function() {
		$("#editCustomNameGroup").show();
		$("#editCustomEigenGebeurtenis").attr("value", "eigenGebeurtenis");
		$("#editCustomHeaderImage").attr('src', "img/CustomEvent.png");
		$("#editCustomHeader").html(jQuery.i18n.prop('CustomEvent'));
	}

});

$(function() {
	/*
	 * Zorg dat de edit pagina niet wordt opgenomen in de browser geschiedenis,
	 * zodat na het invoeren de gebruiker op de back knop kan klikken.
	 */
	$('#editCustomLink').click(function() {
		// Eigen gebeurtenis invoeren, maak titel editable
		navigationController.gotoPage("#editCustom", true);
		eigenGebeurtenisController.New();
	});

	$('#customEventList').on("pagebeforeshow", function() {
		fillCustomEventList();

		/*
		 * Als gebruiker op eigen gebeurtenis type snellijst klikt, dit
		 * afhandelen en invullen op de vorige pagina
		 */
		$("#editCustomEventList > li").click(function() {
			navigationController.gotoPage("#editCustom", true);
			$("#editCustomName").val($("h3", this).text());
		});
	});

	/*
	 * Click afhandelen op eigen gebeurtenis titel-snelkeuze menu
	 */
	createIconLink($('#editCustomName'), $('#customEventListLink'), function() {
		navigationController.gotoPage("#customEventList");
	});

	/*
	 * Eigen bijzonderheden form submission afhandelen
	 */
	$('#editCustomSubmit').on("click", function() {
		var editEigenGebeurtenis = $("#editCustomEigenGebeurtenis").attr("value");

		/*
		 * Omdat we hetzelfde form gebruiken voor 'Eigen gebeurtenis' als voor
		 * 'Algemene gebeurtenis' checken we op de waarde van het hidden field.
		 */
		if (editEigenGebeurtenis == "eigenGebeurtenis") {
			var eventDate = $("#editCustomDatum").val();
			var eventDateEnd = $("#editCustomDatumEnd").val();
			var eventStartTime = $("#editCustomStartTijd").val();
			var eventType = $("#editCustomName").val();
			var eventEindTijd = $("#editCustomEindTijd").val();
			var eventNotes = $("#editCustomNote").val();
			var dateTimeStart = Date.parse(eventDate + " " + eventStartTime);
			var dateTimeEind = Date.parse(eventDateEnd + " " + eventEindTijd);

			if (eventType === "") {
				var message = $.i18n.prop("i18n_validation_error_name_for_event");
				phoneAlert(message);
				return false;
			}
			if (!validator.validateStarttimePreceedsEndTime(dateTimeStart, dateTimeEind)) {
				return false;
			}

			/* EIGEN GEBEURTENIS OPSLAAN */
			/*
			 * De custom event types worden apart opgeslagen in het geval de
			 * gebruiker gebruik maakt van de snelkeuze bij het invoeren van een
			 * eigen type bijzonderheid. Op deze manier kunnen we snel de meest
			 * gebruikte custom eventTypes tonen.
			 * 
			 * Zoek of het event type bestaat en zoek welk eventType de oudste //
			 * is.
			 */
			var eventTypes = getJSON("epEventType");
			var oldestEventTypeId = -1;
			var newEpEventType = null;
			if (eventTypes != null) {
				var oldest = -1; // Zoek oudste id in de lijst
				for ( var i = 0; i < eventTypes.length; i++) {
					if (eventTypes[i].lastUsed < oldest || oldest == -1) {
						// Zoek oudste bestaande eventType
						oldest = eventTypes[i].lastUsed;
						oldestEventTypeId = eventTypes[i].id;
					}
					if (eventTypes[i].eigenTitel == eventType) {
						newEpEventType = eventTypes[i]; // Pak bestaande
						// eventType
					}
				}
			}

			/*
			 * Als het eventType is gevonden, gebruik deze dan. Zo niet, maak
			 * dan een nieuw eventType aan en sla deze up. In alle gevallen het
			 * lastUsed attribuut bijwerken en opslaan.
			 */
			if (newEpEventType != null) {
				newEpEventType.lastUsed = uniqueId();
				storeJSON(newEpEventType, newEpEventType.id);
			} else {
				newEpEventType = new epEventType("CustomEvent");
				newEpEventType.lastUsed = uniqueId();
				newEpEventType.custom = true;
				newEpEventType.eigenTitel = eventType;

				// Oudste moet plaats maken voor nieuwe event.
				if (eventTypes != null && eventTypes.length >= 10) {
					removeJSON("epEventType", oldestEventTypeId);
				}

				storeJSON(newEpEventType);
			}

			// Maak custom event met het juiste eventType
			var customEvent = new epEventCustom(dateTimeStart, newEpEventType);
			customEvent.notes = eventNotes;
			customEvent.eindDateTime = dateTimeEind;

			if (eventController.getAction() == ACTION.Edit) {
				eigenGebeurtenisController.update(customEvent);
			} else {
				eigenGebeurtenisController.save(customEvent);
			}

			eventController.Clear();

			navigationController.gotoPreviousPage();
			return false;
		}
	});
});

/*
 * Dynamisch de eigen bijzonderheden snelkies-lijst vullen. Vullen met
 * epEventType objecten die custom=true als attribute hebben.
 */
function fillCustomEventList() {
	var customEvents = getJSON("epEventType");

	// Gooi alle child items weg en bouw de lijst opnieuw op
	$("#editCustomEventList").empty();

	if (customEvents != null) {
		for ( var i = 0; i < customEvents.length; i++) {
			$("#editCustomEventList").append("<li><a href='#'><h3>" + customEvents[i].eigenTitel + "</h3></a></li>");
		}
	} else {
		// stop een standaard waarde in de lijst
		$("#editCustomEventList").append("<li><a href='#'><h3>" + jQuery.i18n.prop('i18n_empty') + "</h3></a></li>");
	}
	// Vernieuw de dynamisch opgebouwde lijst
	$("#editCustomEventList").listview("refresh");
}
