/**
 * In geval van edit mode, moeten de gegevens van een bestaand event worden
 * bijgewerkt en weer correct worden opgeslagen in de storage.
 */
var eventController = epExtend(controller, {

    /**
    * Override. Als er een nieuw event wordt aangemaakt, verberg dan de delete
    * knop.
    */
    New: function () {
        this._super.New();
        $(".eventDeleteLink").hide();
    },

    /**
    * Override. Als een bestaand event wordt bewerkt, sla de datum van het
    * event dan op zodat we dit event makkelijk kunnen terugvinden in de
    * storage. Tevens wordt de delete knop zichtbaar.
    * 
    * @param event
    *            Het epEvent object (instance) die bewerkt wordt.
    */
    Edit: function (event) {
        this._super.Edit(event);
        var date = Date.parse(event.startDateTime);
        window.sessionStorage.setItem('controllerDate', date.toDateString());
        $(".eventDeleteLink .ui-btn-text").text(jQuery.i18n.prop('i18n_delete_event'));
        $(".eventDeleteLink").show();
    },

    /**
    * Override. Geen super ivm specifieke opslag van events. Verwijder het
    * event wat momenteel wordt ge-edit.
    */
    Delete: function () {
        var eventDate = this.getObjectDate();
        var eventId = this.getObjectId();
        var event = getEventByDay(new Date(eventDate), eventId);

        // omniture: verlaag agenda item count
        var omnitureArr = getJSON('epOmniture');
    	var omnitureInfo = (omnitureArr == null ? new epOmniture() : omnitureArr[0]);
    	omnitureInfo.agendaCount--;
    	storeJSON(omnitureInfo, omnitureInfo.id);
        
        // Datalayer het event laten verwijderen.
        removeEvent(event);
        this.Clear();
    },

    save : function(obj) {
        // omniture: verhoog agenda item count
        var omnitureArr = getJSON('epOmniture');
    	var omnitureInfo = (omnitureArr == null ? new epOmniture() : omnitureArr[0]);
    	omnitureInfo.agendaCount++;
    	storeJSON(omnitureInfo, omnitureInfo.id);
    	
    	this._super.save(obj);
    },
    
    /**
    * Override. Aanroepen nadat het bewerken/nieuw aanmaken is voltooid.
    * Opschonen van de sessionstorage die wordt gebruikt om tijdelijke
    * informatie op te slaan over het epEvent.
    */
    Clear: function () {
        this._super.Clear();
        window.sessionStorage.removeItem('controllerDate'); // to make sure
    },

    /**
    * Opvragen van de datum van het epEvent (instance) die nu wordt bewerkt.
    * 
    * @returns De datum van het event object. Null als deze niet via 'Edit' is
    *          gezet.
    */
    getObjectDate: function () {
        return window.sessionStorage.getItem('controllerDate');
    },

    /**
    * Override. Geen super call ivm andere manier van opslaan van events.
    * Bijwerken van een bestand event. Vanwege de manier van storage (nl.
    * events worden opgeslagen op de datum waarbij ze horen) is het belangrijk
    * om te bepalen of de datum is gewijzigd. De datalayer handelt af wat er
    * moet gebeuren als de datum is bijgewerkt.
    * 
    * @param event
    */
    update: function (event) {
        /*
        * We must use the ID from the controller and not the ID of the passed
        * event, since that ID contains a newly generated ID. So this ID won't
        * match the existing stored object id
        */
        event.id = this.getObjectId();

        var prevDate = this.getObjectDate();
        var newDate = event.startDateTime.toDateString();

        if (prevDate == newDate) {
            // Date was not modified
            storeJSON(event, event.id);
        } else {
            /*
            * Als de datum is gewijzigd moeten we het event opslaan onder een
            * andere key @TODO: Het zou mooier zijn als de controller niet zou
            * weten dat events een datum als key hebben
            */
            updateEventKey(prevDate, newDate, event);
            storeJSON(event, event.id);
        }
    },

    getNewEventDate: function () {
        var currentDateTime = new Date();
        var lastSelectedDate = sessionStorageWrapper().GetSelectedDay();
        if (lastSelectedDate !== null) {
            
            var now = new Date();
            //set time component to current time
            lastSelectedDate.addHours(now.getHours());
            lastSelectedDate.addMinutes(now.getMinutes());
    

            
            return lastSelectedDate;
            
        };

        return currentDateTime;
    }

});

/**
 * Afhandelen van een klik op de algemene 'delete event' knop die wordt getoond
 * als de controller/viewer in Edit mode is. Vraag een bevestiging en verwijder
 * het event bij akkoord.
 */
$('.eventDeleteLink').live('click', function () {
    phoneConfirm(jQuery.i18n.prop('i18n_confirm_delete_event'), function (){
        eventController.Delete();
        navigationController.gotoPreviousPage();
    });
});