/**
 * Controller voor Incidenteel Medicijn Gebruik gebeurtenissen.
 */
var medicijnController = epExtend(eventController, {

	/**
	 * Override. Deze functie wordt aangeroepen wanneer er een nieuw medicijn
	 * moet worden aangemaakt.Form initializeren. Leegmaken van bepaalde velden
	 * en datum/tijd prefillen.
	 */
	New : function() {
		this._super.New();
		var currentDateTime = this._super.getNewEventDate();

		$('#incidentDrugName').add("#incidentDrugQuantity").add("#incidentDrugMissed").not(':submit').val('');
		$("#incidentDrugDate").val(currentDateTime.toDisplayDateString());
		$("#incidentDrugTime").val(currentDateTime.toDisplayTimeString());
		$("#incidentDrugQuantity").val('1');
	},

	/**
	 * Deze functie wordt aangeroepen als een bestaand event wordt bewerkt.
	 * 
	 * @param event:
	 *            Het event object dat bewerkt moet worden.
	 */
	Edit : function(event) {
		this._super.Edit(event);

		navigationController.gotoPage("#editIncidenteleMedicatie");
		var dateTime = Date.parse(event.startDateTime);

		// vul form
		$("#incidentDrugName").val(event.drugName);
		$("#incidentDrugDate").val(dateTime.toDisplayDateString());
		$("#incidentDrugTime").val(dateTime.toDisplayTimeString());
		$("#incidentDrugQuantity").val(event.quantity);
		$("#incidentDrugUnit").val(event.unit);
		$('#incidentDrugUnit').selectmenu('refresh');
	}

});

/**
 * Viewer gedeelte. Afhandelen en registreren van GUI events.
 */
$(function() {

	$('#editIncidenteleMedicatie').on('pageinit', function() {
		// koppelen van button aan drugname veld.
		createIconLink($('#incidentDrugName'), $('#incidentDrugListLink'), function() {
			navigationController.gotoPage("#drugList");
		});
		createQuickMedicineList("#incidentDrugName");
	});

	$('#editIncidenteleMedicateLink').click(function() {
		navigationController.gotoPage("#editIncidenteleMedicatie", true);
		medicijnController.New();
	});

	/*
	 * Incidentele medicatie form submission afhandelen
	 */
	$('#incidentDrugSubmit').on("click", function() {
		var message = "";
		var drugDate = $("#incidentDrugDate").val();
		var drugTime = $("#incidentDrugTime").val();
		var drugName = $("#incidentDrugName").val();
		var dateTime = Date.parse(drugDate + " " + drugTime);

		var newDrug = new epEventIncidentDrug(dateTime, drugName);

		// Retrieve all filled in fields
		newDrug.unit = $("#incidentDrugUnit").val();
		newDrug.quantity = $("#incidentDrugQuantity").val();
		newDrug.missed = $("#incidentDrugMissed").is(":checked");
		newDrug.eindDateTime = dateTime;

		// validate input
		if (newDrug.drugName === "") {
			message = $.i18n.prop("i18n_validation_error_enter_name_for_medication");
			phoneAlert(message);
			return false;
		}

		// numerieke validatie is toch nodig omdat je zelfs met telefone
		// keyboard kan pasten...
		if (!validator.validateStringIsFloatNumberGreaterThanZero(newDrug.quantity)) {
			return false;
		}

		if (eventController.getAction() == ACTION.Edit) {
			eventController.update(newDrug);
		} else {
			eventController.save(newDrug);
		}
		eventController.Clear();

		navigationController.gotoPreviousPage();
		return false;
	});
});

/**
 * @param targetInputId
 *            de id van de inputbox waaraan de quicklistbutton gekoppeld is
 */
function createQuickMedicineList(targetInputId) {
	// vind de parent van de inputbox, omdat we terug willen navigeren.
	var parentPageId = $(targetInputId).closest('div[data-role="page"]').attr("id");
	/*
	 * Als er een medicijn uit de snelkeuze lijst is geselecteerd, vul de waarde
	 * dan in op de vorige pagina en navigeer daar naar terug.
	 */
	$("#incidentDrugList > li").unbind('click'); //

	/*
	 * nodig omdat anders de oude clickevents blijven (kan een andere pagina
	 * zijn)
	 */
	$("#incidentDrugList > li").on('click', function() {
		navigationController.gotoPage("#" + parentPageId, true);
		$(targetInputId).val($("p", this).text());
	});
}
