var mijnMedicatieController = epExtend(controller, {

	/**
	 * Override. Nieuwe medicatie toevoegen.
	 */
	New : function() {
		this._super.New();
		// clear session and set to new
		this.clearSession();

		// clear/init form
		$("#myDrugName").val('');
		// $("#drugUnit").selectmenu('refresh');
		$('#notifyMedicineSwitch').prop("checked", true).checkboxradio("refresh");

		$(".medicineDeleteLink").hide();
		this.buildDayPartsList();
	},

	/**
	 * Override. Bestaande medicatie bewerken
	 * 
	 * @param medicine
	 */
	Edit : function(medicine) {
		this._super.Edit(medicine);

		// fill form with recurring event
		$("#myDrugName").val(medicine.drugName);
		$("#drugUnit").val(medicine.unit).selectmenu("refresh", true);
		$('#notifyMedicineSwitch').prop("checked", medicine.sendNotification).checkboxradio("refresh");

		$(".medicineDeleteLink .ui-btn-text").text(jQuery.i18n.prop('i18n_delete_medicine'));
		$(".medicineDeleteLink").show();
		this.buildDayPartsList(medicine);
	},

	/**
	 * Override. Maar geen call naar super omdat deze update andere
	 * functionaliteit moet hebben.
	 * 
	 * @param med
	 */
	update : function(med) {
		/*
		 * We must use the ID from the controller and not the ID of the passed
		 * event, since that ID contains a newly generated ID. So this ID won't
		 * match the existing stored object id
		 */
		med.id = this.getObjectId();
		storeJSON(med, med.id);
	},

	clearSession : function() {
		for (daypartname in EP_DAY_PART) {
			window.sessionStorage.removeItem(daypartname);
		}
	},

	/**
	 * Het aanmaken van de mijn medicatie hoofdpagina. Alle medicijnen ophalen en
	 * tonen.
	 */
	createMijnMedicatie : function() {
		var customMedicines = this.getAll();
		var artsBehandelschema = behandelschemaController.get();
		if (artsBehandelschema == undefined) {
			this.buildMedicijnenList(customMedicines);
		} else {
			this.buildMedicijnenList(customMedicines, artsBehandelschema.medicijnSchemas);
		}
	},

	/**
	 * Vullen van een dagdeel listitem
	 * 
	 * @param daypart
	 *           epMedicineDayPart object
	 */
	createDayPartPage : function(daypart) {
		// clear form
		$("#dayPartPage").find('.dayPartIcon').attr('src', 'img/' + daypart.daypart + '.png');
		$("#dayPartPage").find('.dayPartTitle').html(jQuery.i18n.prop("i18n_title_" + daypart.daypart));

		$("#dayPartTimeHidden").val(daypart.time);
		$("#dayPartQuantity").val(daypart.quantity);
		$('#dayPartPage').attr('day', daypart.daypart);
		$('#clearDaypartLink').attr('day', daypart.daypart);
	},

	/**
	 * Bouwt de lijst met dagdelen op in de nieuwe/edit medicijn pagina. Als een
	 * medicijn wordt meegegeven wordt deze ingevuld.
	 * 
	 * @param medicine
	 * 
	 */
	buildDayPartsList : function(medicine) {
		$('#lvDayPart').empty();
		var i = 0;
		/*
		 * alle dagdelen doorlopen. Als een dagdeel niet gekoppeld kan worden aan
		 * een localstorage waarde of een session waarde, geef deze dan leeg en
		 * transparant weer. Dagdelen worden vÃ³Ã³r het opslaan alleen opgeslagen
		 * in de sessie.
		 */
		for (time_of_day in EP_DAY_PART) {
			var $template = $('.dayPart-template').clone().removeClass('dayPart-template');
			$template.addClass("dayPartListItem");
			/*
			 * eerst het dagdeel van het medicijn (indien meegegeven), anders uit
			 * de sessie
			 */
			var daypart = undefined;
			var daypartSession = this.getDaypart(time_of_day);
			if (medicine != undefined && medicine.dayparts) {
				// alert(medicine);
				/*
				 * dayparts is een array van dagdelen, doorzoek de array naar het
				 * juiste dagdeel.
				 */
				for ( var p in medicine.dayparts) {
					if (medicine.dayparts[p].daypart == time_of_day) {
						daypart = medicine.dayparts[p];
						break;
					}
				}
			} else if (daypartSession && daypartSession != null) {
				daypart = daypartSession;
			}
			i++;
			if (daypart) {
				this.setDaypart(daypart); // set it in the session too.
				$template.find('.dayPartTime').html(daypart.time);
				var translatedUnit = jQuery.i18n.prop(getEnumByValue(EP_EVENT_INCIDENT_DRUG_UNIT, $("#drugUnit").val()));
				var translatedDaypart = jQuery.i18n.prop("i18n_title_" + daypart.daypart);
				var dayPartText = jQuery.i18n.prop('i18n_daypart_text', daypart.time, translatedDaypart, daypart.quantity, translatedUnit);
				$template.removeClass('transparent').find('.dayPartDescription').html(dayPartText);
			} else {
				$template.addClass('transparent');
			}
			$template.attr('day', time_of_day);
			$template.find('.dayPartLink').attr('day', time_of_day);
			$template.find('.dayPartIcon').attr('src', 'img/' + time_of_day + '.png');
			$template.find('.dayPartTitle').html(jQuery.i18n.prop("i18n_title_" + time_of_day));

			$('#lvDayPart').append($template.show());
		}
		$('#lvDayPart').listview('refresh');
	},

	// updating is faster than fully redrawing
	updateDayPartsList : function(daypart) {

		if (daypart) {
			var translatedUnit = jQuery.i18n.prop(getEnumByValue(EP_EVENT_INCIDENT_DRUG_UNIT, $("#drugUnit").val()));
			var translatedDaypart = jQuery.i18n.prop("i18n_title_" + daypart.daypart);
			var dayPartText = jQuery.i18n.prop('i18n_daypart_text', daypart.time, translatedDaypart, daypart.quantity, translatedUnit);
			$('#lvDayPart > li[day="' + daypart.daypart + '"]').removeClass('transparent').find('.dayPartDescription').html(dayPartText);
		}

		$('#lvDayPart').listview('refresh');
	},

	clearDayPartsList : function(daypart_name) {
		$('#lvDayPart > li[day="' + daypart_name + '"]').addClass('transparent').find('.dayPartDescription').html("");
	},

	setDaypart : function(daypart) {
		window.sessionStorage.setItem(daypart.daypart, JSON.stringify(daypart));
	},

	getDaypart : function(daypart_name) {
		var sessionDaypart = window.sessionStorage.getItem(daypart_name);
		return (sessionDaypart != null) ? JSON.parse(sessionDaypart) : undefined;
	},

	removeDaypart : function(daypart_name) {
		window.sessionStorage.removeItem(daypart_name);
		this.clearDayPartsList(daypart_name);
	},

	getAll : function() {
		return getJSON('epRecurringMedicine');
	},

	get : function(id) {
		return getJSON('epRecurringMedicine', id);
	},

	remove : function(id) {
		// first remove any notifications for this medicine
		var medicine = getJSON('epRecurringMedicine', id);
		mijnMedicatieController.removeNotification(medicine);

		removeJSON('epRecurringMedicine', id);
	},
	
	/*
	 * Opbouwen medicijnenlijst op de hoofdpagina met template. Vult de
	 * beschrijving van de medicijnen zodat duidelijk is in een oogopslag wat er
	 * genomen moet worden.
	 */
	buildMedicijnenList : function(customMedicines, behandelschemaMedicines) {
		$('#myDrugList').empty();

		for ( var m in customMedicines) {
			var med = customMedicines[m];
			var $template = $('.medicine-template').clone().removeClass('medicine-template');
			$template.find('.medicineTitle').html(med.drugName);
			$template.addClass("medicineListItem");
			$template.find('.medicineLink').attr('id', med.id);
			var medicineUnit = jQuery.i18n.prop(getEnumByValue(EP_EVENT_INCIDENT_DRUG_UNIT, med.unit));
			// description vullen met gegevens over tijden en hoeveelheden.
			var description = [];
			description.push(jQuery.i18n.prop('i18n_medicine_scheme_custom'), " ");

			/* slice verwijdert de laatste comma */
			$template.find('.medicineDescription').html(description.join('').slice(0, -1));

			$('#myDrugList').append($template.show());
		}

		for ( var m in behandelschemaMedicines) {
			var schema = behandelschemaMedicines[m];
			var $template = $('.medicine-template').clone().removeClass('medicine-template');
			$template.find('.medicineTitle').html(schema.drugName);
			$template.addClass("medicineListItem");
			$template.find('.medicijnIcon').attr('src', 'img/Behandelschema.png');
			$template.find('.medicineLink').attr('class', 'behandelschemaLink').attr('id', schema.id);

			var medicineUnit = jQuery.i18n.prop(getEnumByValue(EP_EVENT_INCIDENT_DRUG_UNIT, schema.unit));

			$template.find('.medicineDescription').html(jQuery.i18n.prop('i18n_medicine_scheme_external'));

			$('#myDrugList').append($template.show());
		}

		$('#myDrugList').listview('refresh');
	},

	createAndSendMedicineRequestEmail : function() {
		var html = [];
		html.push(jQuery.i18n.prop('i18n_medicineRequest_aanhef'), "<br>");
		html.push('<h3>', jQuery.i18n.prop('i18n_report_medicijnen_overzicht'), '</h3>');
		var medicines = mijnMedicatieController.getAll();
		for (i in medicines) {
			var med = medicines[i];
			html.push(med.drugName, "\t\t\t\t", "<br>");
		}

		var behandelschema = behandelschemaController.get();
		if (behandelschema != null) {
			for (i in behandelschema.medicijnSchemas) {
				var schema = behandelschema.medicijnSchemas[i];
				html.push(schema.drugName, "\t\t\t\t", "<br>");
			}
		}

		// send email
		var subject = jQuery.i18n.prop('i18n_medicineRequest_subject');

		QuintorPhonegapPlugins.sendEmail(subject, html.join(''));
	},
	setNotification : function(medicine) {

		// enable
		for (i in medicine.dayparts) {
			var dp = medicine.dayparts[i];
			var message = jQuery.i18n.prop('i18n_medicine_notification_message');
			var notificationId = medicine.id + '-' + i;
			var medicineUnit = jQuery.i18n.prop(getEnumByValue(EP_EVENT_INCIDENT_DRUG_UNIT, medicine.unit));
			// enabled
			if (medicine.sendNotification) {
				NotificationService.addDailyNotification(notificationId, dp.time, message + ": \r\n" + dp.quantity + " " + medicineUnit + " - "
						+ medicine.drugName);
			} else {
				NotificationService.removeNotification(notificationId);
			}
		}

	},
	removeNotification : function(medicine) {
		if (medicine.sendNotification) {
			for (i in medicine.dayparts) {
				var notificationId = medicine.id + '-' + i;
				NotificationService.removeNotification(notificationId);
			}
		}
	}

});

$(function() {
	$('#dayPartSubmit').on("click", function() {
		var time_of_day = $('#dayPartPage').attr('day');
		var time = $('#dayPartPage').find('#dayPartTimeHidden').val();
		var quantity = $('#dayPartPage').find('#dayPartQuantity').val();
		var daypart = new epMedicineDayPart(time_of_day, time, quantity);

		if (!validator.validateStringIsFloatNumberGreaterThanZero(quantity)) {
			return false;
		}
		mijnMedicatieController.setDaypart(daypart);
		mijnMedicatieController.updateDayPartsList(daypart);
		return true;
	});

	$('.medicineDeleteLink').live('click', function() {
		phoneConfirm($.i18n.prop('i18n_confirm_delete_medicine'), function() {
			var medicineId = mijnMedicatieController.getObjectId();

			mijnMedicatieController.remove(medicineId);
			mijnMedicatieController.clearSession();

			navigationController.gotoPage("#mijnMedicijnenPage");
		});
	});

	/*
	 * Click afhandelen in de dayparts list. Popup met instellen tijd en popup
	 * met instellen quantity. Daarna invullen in de partslist
	 */
	$(".dayPartLink").live("click", function() {
		navigationController.gotoPage("#dayPartPage");

		var daypart = mijnMedicatieController.getDaypart($(this).attr('day'));
		daypart = daypart || new epMedicineDayPart($(this).attr('day'));
		mijnMedicatieController.createDayPartPage(daypart);

	});

	$('#newMedicatieLink').click(function() {
		navigationController.gotoPage("#mijnMedicijnenEditPage", true);
		mijnMedicatieController.New();
	});

	$('#mijnMedicijnenEditPage').on('pageinit', function() {
		// koppelen van button aan drugname veld.
		createIconLink($('#myDrugName'), $('#drugListLink'), function() {
			navigationController.gotoPage("#drugList");
			createQuickMedicineList("#myDrugName");
		});
	});

	$("#mijnMedicijnenPage").on("pagebeforeshow", function() {
		mijnMedicatieController.createMijnMedicatie();
	});

	/*
	 * edit link voor het bekijken van een behandelschema. Het id an de link
	 * verwijst naar het serverid van het schema.
	 */
	$(".behandelschemaLink").live("click", function() {
		navigationController.gotoPage("#behandelschemaEditPage");

		var serverId = $(this).attr('id');
		behandelschemaController.Edit(serverId);
	});

	/*
	 * view link voor het bekijken van de details van een medicijn. Het id an de
	 * link verwijst naar het item in storage.
	 */
	$(".medicineLink").live("click", function() {
		navigationController.gotoPage("#mijnMedicijnenEditPage", true);

		var medicine = getJSON('epRecurringMedicine', $(this).attr('id'));
		mijnMedicatieController.Edit(medicine);
	});

	/*
	 * Maakt een dagdeel weer transparant en leegt de tekst.
	 */
	$('#clearDaypartLink').on('tap', function() {
		navigationController.gotoPreviousPage();
		// "#mijnMedicijnenEditPage", {
		// changeHash: false
		// });
		mijnMedicatieController.removeDaypart($(this).attr('day'));
	});

	/*
	 * nieuwe medicatie form submission afhandelen
	 */
	$('#editDrugSubmit').live("click", function() {
		var dayPartDefined = false;
		var message = "";
		console.log("medicineLIST");
		var medicine = new epRecurringMedicine();
		medicine.drugName = $("#myDrugName").val();
		medicine.unit = $("#drugUnit").val();
		medicine.sendNotification = $('#notifyMedicineSwitch').is(':checked');

		if (medicine.drugName === "") {
			message = $.i18n.prop("i18n_validation_error_medicine_name");
			phoneAlert(message);
			return false;
		}

		for (daypart in EP_DAY_PART) {
			var daypartObject = mijnMedicatieController.getDaypart(daypart);
			if (daypartObject) {
				dayPartDefined = true;
				medicine.dayparts.push(daypartObject);
			}
		}

		// check we have defined at least one daypart
		if (!dayPartDefined) {
			message = $.i18n.prop("i18n_validation_error_medicine_no_daypart");
			phoneAlert(message);
			return false;
		}

		if (eventController.getAction() == ACTION.Edit) {
			mijnMedicatieController.update(medicine);
		} else {
			console.log('drug saved');
			mijnMedicatieController.save(medicine);
		}

		mijnMedicatieController.setNotification(medicine);
		mijnMedicatieController.clearSession();

		navigationController.gotoPage("#mijnMedicijnenPage");

		return false;
	});

	$('#bestelMedicijnen').on("click", function() {
		mijnMedicatieController.createAndSendMedicineRequestEmail();
		behandelschemaController.setTestdata();
	});

	$('#cancelAllNotifications').on("click", function() {
		//
		var message = $.i18n.prop("i18n_confirm_remove_notification");
		phoneConfirm(message, function() {
			// 1 zet op alle medicijnen notificaties uit
			var allMedication = mijnMedicatieController.getAll();
			for (i in allMedication) {
				var medicine = allMedication[i];
				medicine.sendNotification = false;
				// kan geen update gebruiken omdat deze het id uit de controller
				// haalt.
				storeJSON(medicine, medicine.id);
				// mijnMedicatieController.update(medicine);
			}
                     
			// 2 zet alle notifications voor behandelschema uit
			var behandelschema = behandelschemaController.get();
            
            if(behandelschema) {
                for (var s in behandelschema.medicijnSchemas) {
                    behandelschema.medicijnSchemas[s].sendNotification = false;
                }
                storeJSON(behandelschema, behandelschema.id);
            }
                     
			// 3 zet alle notificaties die gescheduled zijn uit
			NotificationService.cancelAllNotifications();
			phoneAlert($.i18n.prop("i18n_all_notifications_removed"));
		});
	});

	$("#mijnMedicatieDetailsLink").live("click", function() {
		navigationController.gotoPage("#medicatieDetailsPage");
		var medId = $(this).attr('medicatieId');
		if (medId && medId != null) {
			mijnMedicatieDetailsController.ViewMijnMedicatie(medId);
		}
	});



});