var serviceController = {

	/**
	 * stuurt het json request naar de server
	 * 
	 * @param endpoint
	 *            specifieke endpoint dat je aanroept
	 * @param jsonObject
	 *            het object dat je wil sturen
	 * @param onSuccess
	 *            welke functie moet worden uitgevoerd als de actie slaagt.
	 */
	sendRequest : function(endpoint, action, jsonObject, onSuccess, onError, onComplete) {
		if (SERVICE_DEBUG) {
			console.log("sending POST request to: " + EP_SERVICE_BASE_URL + endpoint + action);
		}
		
		var settings = getSettings();
		jsonObject.language = settings.language;
		
		$.ajax({
			type : "POST",
			url : EP_SERVICE_BASE_URL + endpoint + action,
			data : JSON.stringify(jsonObject),
			dataType : "json",
			cache : false,
			contentType : "application/json",
			beforeSend : serviceController.beforeSend,
			success : onSuccess || serviceController.syncSuccess,
			error : onError || serviceController.syncError,
			complete : onComplete || serviceController.syncComplete
		});
	},

	getRequest: function(endpoint, action, onSuccess, onError, onComplete) {
		if (SERVICE_DEBUG) {
			console.log("sending GET request to: " + EP_SERVICE_BASE_URL + endpoint + action);
		}
		$.ajax({
			type : "GET",
			url : EP_SERVICE_BASE_URL + endpoint + action,
			dataType : "json",
			cache : false,
			beforeSend : serviceController.beforeSend,
			success : onSuccess || serviceController.syncSuccess,
			error : onError || serviceController.syncError,
			complete : onComplete || serviceController.syncComplete
		});
	},
	
	beforeSend : function(xhr) {
		var account = accountController.getOrCreateAccount();
		
		if( account != null ) {
			var authString = account.username + ":" + hex_sha256( account.password + "{" + EP_PASSWORD_SALT + "}");
			var authBase64 = Base64.encode(authString);
			xhr.setRequestHeader("Authorization", "Basic " + authBase64 );
		} else {
			return;
		}

	},

	/**
	 * Error callback die wordt aangeroepen als de ajax post naar de server
	 * mislukt.
	 * 
	 * @param jqXHR
	 */
	syncError : function(jqXHR) {
		if (SERVICE_DEBUG) {
			// console.log("syncError: " + JSON.stringify(jqXHR));
			console.log('sync error');
		}
	},

	/**
	 * Callback die wordt aangeroepen naar de ajax post naar de server.
	 * 
	 * @param jqXHR
	 */
	syncComplete : function(jqXHR) {
		if (SERVICE_DEBUG) {
			console.log('sync complete');
			// console.log("syncComplete: " + JSON.stringify(jqXHR));
		}
	},

	/**
	 * Callback die wordt aangeroepen naar de ajax post naar de server.
	 * 
	 * @param jqXHR
	 */
	syncSuccess : function(jqXHR) {
		if (SERVICE_DEBUG) {
			console.log('sync success');
			// console.log("syncSuccess: " + JSON.stringify(jqXHR));
		}
	}

};

var Base64 = {

	// private property
	_keyStr : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

	// public method for encoding
	encode : function(input) {
		var output = "";
		var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
		var i = 0;

		input = Base64._utf8_encode(input);

		while (i < input.length) {

			chr1 = input.charCodeAt(i++);
			chr2 = input.charCodeAt(i++);
			chr3 = input.charCodeAt(i++);

			enc1 = chr1 >> 2;
			enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
			enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
			enc4 = chr3 & 63;

			if (isNaN(chr2)) {
				enc3 = enc4 = 64;
			} else if (isNaN(chr3)) {
				enc4 = 64;
			}

			output = output + this._keyStr.charAt(enc1) + this._keyStr.charAt(enc2) + this._keyStr.charAt(enc3)
					+ this._keyStr.charAt(enc4);

		}

		return output;
	},
	
	// private method for UTF-8 encoding
	_utf8_encode : function (string) {
	    string = string.replace(/\r\n/g,"\n");
	    var utftext = "";

	    for (var n = 0; n < string.length; n++) {

	        var c = string.charCodeAt(n);

	        if (c < 128) {
	            utftext += String.fromCharCode(c);
	        }
	        else if((c > 127) && (c < 2048)) {
	            utftext += String.fromCharCode((c >> 6) | 192);
	            utftext += String.fromCharCode((c & 63) | 128);
	        }
	        else {
	            utftext += String.fromCharCode((c >> 12) | 224);
	            utftext += String.fromCharCode(((c >> 6) & 63) | 128);
	            utftext += String.fromCharCode((c & 63) | 128);
	        }

	    }

	    return utftext;
	},
};
