var DEFAULT_KEY = 'standard_key';
var DATALAYER_LOGGING = false;

/*
 * Cache variable die wordt bijgewerkt wanneer er naar de local storage wordt
 * geschreven. Dit voorkomt dat de maand calender 30x deze variable door moet.
 * 
 * Let er op dat deze cache wordt bijgewerkt als een multiday event verandert!!
 */
var EP_MULTIDAY_CACHE = null;

// / Wrapper 'class' around window.sessionStorage. Use like
// sessionStorageWrapper().Get...
function sessionStorageWrapper() {
	var SS_SelectedDay = "selected_day";
	var SS_ControllerAction = "controllerAction";
	var SS_ControllerObject = "controllerObject";
	var SS_ControllerObjectType = "controllerObjectType";

	var get = function(key) {
		return window.sessionStorage.getItem(key);
	};
	var set = function(key, value) {
		window.sessionStorage.setItem(key, value);
	};
	var remove = function(key) {
		window.sessionStorage.removeItem(key);
	};

	return {
		GetSelectedDay : function() {
			var s = get(SS_SelectedDay);
			if (s !== null) {
				return new Date(s);
			} else {
				return null;
			}
		},
		SetSelectedDay : function(date) {
			if (date !== null) {
				set(SS_SelectedDay, date.toDateString());
			} else {
				set(SS_SelectedDay, null);
			}
		},
		GetControllerAction : function() {
			return get(SS_ControllerAction);
		},
		SetControllerAction : function(action) {
			set(SS_ControllerAction, action);
		},
		RemoveControllerAction : function() {
			remove(SS_ControllerAction);
		},
		GetControllerObject : function() {
			return get(SS_ControllerObject);
		},
		SetControllerObject : function(objId) {
			set(SS_ControllerObject, objId);
		},
		RemoveControllerObject : function() {
			remove(SS_ControllerObject);
		},
		GetControllerObjectType : function() {
			return get(SS_ControllerObjectType);
		},
		SetControllerObjectType : function(type) {
			set(SS_ControllerObjectType, type);
		},
		RemoveControllerObjectType : function() {
			remove(SS_ControllerObjectType);
		}
	};
}

/**
 * Encrypt a given string.
 * 
 * @param originalString
 *            The string that should be encrypted
 * @param key
 *            Optional. If omitted the key from sessionstorage will be used.
 * @returns Encrypted version of the originalString.
 */
function aesEncrypt(originalString, key) {
	if (key == undefined) {
		key = window.sessionStorage.getItem('key');
	}
	return GibberishAES.enc(originalString, key);
	
};

/**
 * Decrypt a given string.
 * 
 * @param encryptedString
 *            The string that must be decrypted.
 * @param key
 *            Optional. If omitted the key from sessionstorage will be used.
 * @returns The original unencrypted version of encryptedString
 */
function aesDecrypt(encryptedString, key) {
	if (key == undefined) {
		key = window.sessionStorage.getItem('key');
	}
	return GibberishAES.dec(encryptedString, key);
};

/**
 * Updates all items in local storage, except 'epSettings' and applies a new
 * encryption key.
 * 
 * @param oldKey
 * @param newKey
 */
function reencodeData(oldKey, newKey) {
	var storageCount = window.localStorage.length;
	for ( var i = 0; i < storageCount; i++) {
		var currentKey = window.localStorage.key(i);
		if (currentKey != "epSettings" && currentKey != "epGeo") {
			try {
				var currentObj = aesDecrypt(window.localStorage.getItem(currentKey), oldKey);

				// Now we have the decrypted object, put them back using the new
				// key
				var newObj = window.localStorage.setItem(currentKey, aesEncrypt(currentObj, newKey));
			} catch (err) {
				console.log("Datalayer.js: unable to reencode the stored data, skipping key: " + currentKey);
			}

		}
	}
}

/**
 * Zoekt naar een object in de lijst onder 'key'.
 * 
 * @param epBaseInstanceArray
 *            Een array met epBase child objecten. Elk epBase object heeft een
 *            id.
 * @param objId
 *            Het id van het object in epBaseInstanceArray
 * 
 * @return arrayIndex wanneer objId is gevonden, anders -1
 */
function getIndexForId(epBaseInstanceArray, objId) {
	if (epBaseInstanceArray == null || epBaseInstanceArray == undefined)
		return -1;

	var found = false;
	for ( var i = 0; i < epBaseInstanceArray.length; i++) {
		if (epBaseInstanceArray[i].id == objId) {
			return i;
		}
	}
	return -1;
};

/**
 * Store JSON object naar storage.
 * 
 * Alle javascript objecten die worden opgeslagen in de local storage moeten via
 * deze methode lopen. Deze functie moet zodanig zijn dat deze in de toekomst de
 * localstorage zou kunnen vervangen door een database of een andere opslag
 * methode.
 * 
 * @param objValue:
 *            Een child van epBase (zie datamodel.js)
 * @param objId:
 *            Optioneel. Wanneer deze wordt meegegeven wordt er gezocht naar
 *            deze key en wordt objValue geupdate.
 * @return False, wanneer objId niet kan worden gevonden. Anders true
 */
function storeJSON(objValue, objId) {

	var objType = objValue.objectType;

	/*
	 * Wrap epEvent instances in an array wrapper that holds all the events for
	 * that specific day.
	 * 
	 * All other epBase instances are stored in an array that holds all epBase
	 * instances of the same type. E.g. all epContact instances are stored in
	 * one list. The 'objectType' field is used as a key to obtain the list.
	 */
	if (objType != undefined) {
		if (objType == "epEvent") {
			var objKey = objValue.startDateTime.toDateString();

			var eventIsMultiDay = isMultiDay(objValue);
			var storedAsMultiDay = getIndexForId(EP_MULTIDAY_CACHE, objId) != -1;

			if (!(objId == undefined || objId == null)) {
				// Bijwerken van een bestaand event.

				if (storedAsMultiDay && eventIsMultiDay) {
					// bijwerken multiday
					return updateObject(EP_MULTI_DAY, objValue, objId);
				} else if (!storedAsMultiDay && !eventIsMultiDay) {
					// bijwerken singleday
					return updateObject(objKey, objValue, objId);
				} else if (storedAsMultiDay && !eventIsMultiDay) {
					// Verplaatsen naar 'start datum' key
					return updateEventKey(EP_MULTI_DAY, objKey, objValue);
				} else if (!storedAsMultiDay && eventIsMultiDay) {
					// Verplaatsen naar multiday key
					return updateEventKey(objKey, EP_MULTI_DAY, objValue);
				}

			} else {
				// Event is nieuw

				if (eventIsMultiDay)
					objKey = EP_MULTI_DAY;

				return wrapAndStore(objKey, objValue);
			}
		} else {
			// Opslaan van een non event

			if (!(objId == undefined || objId == null)) {
				return updateObject(objType, objValue, objId);
			} else {
				if (objType == "epSettings" || objType == "epGeo") {
					// do not encrypt epSettings
					return wrapAndStore(objType, objValue, false);
				}
				return wrapAndStore(objType, objValue);
			}
		}
	} else {
		if (DATALAYER_LOGGING)
			console.log('datalayer.storeJSON: can only store valid instances of epBase');
	}

	return true;
}

/**
 * Haal een JSON object uit de localstorage
 * 
 * @param key
 *            De key waar onder het JSON object is opgeslagen. Voor epEvent
 *            children is de key de datum van de epEvent instance. Voor alle
 *            andere epBase instances, wordt de epBase.objectType als key
 *            gebruikt.
 * 
 * @param objId
 *            Optioneel. Wanneer mee gegeven wordt er gezocht in 'key' naar een
 *            object met id=objId
 * 
 * @returns Als objId undefined is, een array met epBase instances. Als objId is
 *          gevonden wordt het epBase instance object terug gegeven. Anders
 *          null.
 */
function getJSON(key, objId) {
	var itemArray = window.localStorage.getItem(key);

	if (itemArray != null) {
		if (key == "epSettings" || key == "epGeo") {
			try {
				itemArray = JSON.parse(itemArray);
			} catch (err) {
				if (DATALAYER_LOGGING)
					console.log("Datalayer: Unable to decrypt/parse value for key: " + key);

				// removing epSettings
				window.localStorage.removeItem("epSettings");
				return null;
			}
		} else {
			try {
				itemArray = JSON.parse(aesDecrypt(itemArray));
			} catch (err) {
				if (DATALAYER_LOGGING)
					console.log("Datalayer: Unable to decrypt/parse value for key: " + key);
				return null;
			}
		}

		if (objId == undefined) {
			// return hele array onder 'key'
			return itemArray;
		} else {
			// return object met id=objId onder 'key'
			var index = getIndexForId(itemArray, objId);
			if (index != -1)
				return itemArray[index];
			else
				return null;
		}
	} else {
		if (DATALAYER_LOGGING)
			console.log("datalayer.getJSON: key not found: " + key);
		return null;
	}
}

/**
 * Verwijdert een JSON object uit de storage. Als dit het laatste object onder
 * de key, wordt de hele key verwijderd.
 * 
 * @param key
 *            De gebruikte key voor het JSON object
 * @param id
 *            Het id van het epBase object dat in de array opgeslagen zit. Als
 *            id=undefined wordt de hele key weggegooid! Als id niet wordt
 *            gevonden, gebeurt er niks.
 * 
 */
function removeJSON(key, id) {
	if (id == undefined) {
		window.localStorage.removeItem(key);
	} else {
		var wrapperArray = JSON.parse(aesDecrypt(window.localStorage.getItem(key)));
		for ( var i in wrapperArray) {
			if (DATALAYER_LOGGING)
				console.log(wrapperArray[i].id);
			if (wrapperArray[i].id == id) {
				wrapperArray.splice(i, 1);
				break;
			}
		}

		if (wrapperArray.length > 0) {
			writeToLocalStorage(key, wrapperArray, true);
		} else {
			window.localStorage.removeItem(key);
		}
	}
}

/**
 * Update the key for an existing event
 * 
 * @param oldKey
 *            De oude key waaronder 'object' te vinden was
 * @param newKey
 *            De nieuwe key waaronder 'object' terecht moet komen
 * @param object
 *            De epEvent instance wat moet worden verplaatst
 * @return true als succesvol, anders false.
 */
function updateEventKey(oldKey, newKey, object) {
	var oldValues = getJSON(oldKey);

	if (oldValues != null) {
		// Zoek object onder oude key
		for ( var i = 0; i < oldValues.length; i++) {
			if (oldValues[i].id == object.id) {
				// gevonden; verplaats object naar nieuwe key
				if (isMultiDay(object))
					newKey = EP_MULTI_DAY;
				var result = wrapAndStore(newKey, object);
				if (result) {
					// opslaan ging goed. Nu verwijderen uit oude key
					removeJSON(oldKey, object.id);
				}
				refreshMultiDayCache();
				return result;
			}
		}
	}
	return false;
}

/**
 * This function takes a key and a JavaScript object.
 * 
 * It checks the localstorage to see if data was already stored under the given
 * key. If not, the data is wrapped in an array and stored back If a key was
 * found, the array is obtained, extended with the given object and stored back.
 */
function wrapAndStore(key, object, encryptObject) {
	var objArray = getJSON(key);

	if (objArray == null) {
		// Er is nog geen lijst voor dit type
		objArray = [];
		objArray.push(object);
		writeToLocalStorage(key, objArray, encryptObject);
		return true;
	} else {
		// Er is wel een lijst, zoek of er al een object is met object.id
		if (getIndexForId(objArray, object.id) == -1) {
			objArray.push(object);
			writeToLocalStorage(key, objArray, encryptObject);
			return true;
		} else {
			if (DATALAYER_LOGGING)
				console.log("There is already an object with id: " + object.id + " for key: " + key);
			return false;
		}
	}
}

/**
 * Private helper function used by wrapAndStore.
 * 
 * @param key
 *            The key under which the object will be stored
 * @param objArray
 *            The array that will hold all objects
 * @param encryptObject
 *            True if the objArray should be encrypted.
 */
function writeToLocalStorage(key, objArray, encryptObject) {
	var jsonString = JSON.stringify(objArray);
	if (key == "epSettings" || key == "epGeo") {
		encryptObject = false;
	}

	if (encryptObject != undefined && encryptObject == false) {
		window.localStorage.setItem(key, jsonString);
	} else {
		window.localStorage.setItem(key, aesEncrypt(jsonString));
	}

	if (key == EP_MULTI_DAY)
		refreshMultiDayCache();
}

function refreshMultiDayCache() {
	EP_MULTIDAY_CACHE = getJSON(EP_MULTI_DAY);
}

/**
 * Geeft de multiday events terug waarvan 'day' tussen de start en einddatum van
 * het multi day event ligt.
 * 
 * @param day
 *            DateJS Date object die in het bereik van dit event moet liggen.
 * @returns Een array met events waarvan het bereik matcht met day
 */
function getMultiDayEventsFor(day) {
	var multiDayEvents = EP_MULTIDAY_CACHE;
	var result = [];

	for ( var md in multiDayEvents) {
		var mdStartDate = Date.parse(multiDayEvents[md].startDateTime);
		var mdEndDate = Date.parse(multiDayEvents[md].eindDateTime);

		if (day.between(new Date(mdStartDate.toDateString()), new Date(mdEndDate.toDateString()))) {
			result.push(multiDayEvents[md]);
		}
	}
	return result;
}

function isMultiDay(event) {
	if (event.eindDateTime == undefined || event.eindDateTime == null) {
		return false;
	}
	var sd = event.startDateTime;
	var ed = event.eindDateTime;
	var startDate = (sd instanceof Date) ? sd.toDateString() : Date.parse(event.startDateTime).toDateString();
	var endDate = (ed instanceof Date) ? ed.toDateString() : Date.parse(event.eindDateTime).toDateString();

	if (event.eindDateTime != undefined && (startDate != endDate))
		return true;
	else
		return false;
}

/**
 * Zoekt een event op, ook multiday events.
 * 
 * @param date
 *            Een date.toDateString() formaat
 * @param id
 *            Het id van het event.
 */
function getEventByDay(date, id) {
	// Kijk op de dag van het event
	var result = getJSON(date.toDateString(), id);
	if (result == null) {
		// Zoek verder in multiday events
		mdResults = getMultiDayEventsFor(date);
		for (i in mdResults) {
			if (mdResults[i].id == id)
				return mdResults[i];
		}
	}
	return result;
}

function removeEvent(event) {
	if (event == null || event == undefined)
		return false;

	if (isMultiDay(event)) {
		removeJSON(EP_MULTI_DAY, event.id);
		
		// Bijwerken multiday cache
		refreshMultiDayCache();
	} else {
		var startDay = Date.parse(event.startDateTime).toDateString();
		removeJSON(startDay, event.id);
	}
}

/**
 * Update object met 'id' in de lijst die onder 'key' hangt.
 */
function updateObject(key, object, objId) {
	var objArray = getJSON(key);

	if (objArray == null) {
		if (DATALAYER_LOGGING)
			console.log("Could not update object with id: " + objId + " because there is no key: " + key);
		return false; // Kan niet updaten als het object niet bestaat
	} else {
		var objIndex = getIndexForId(objArray, objId);
		if (objIndex != -1) {
			objArray[objIndex] = object; // Vervang object
			writeToLocalStorage(key, objArray, true);
			return true;
		} else {
			if (DATALAYER_LOGGING)
				console.log("Could not update object with id: " + objId + " for localstorage key: " + key);
			return false;
		}
	}
}