/*
 * ENUMS/CONSTANTS used by objects of the datamodel
 */
var EP_BUILD = 18;
var EP_EVENT_INCIDENT_DRUG_UNIT = new Enum([ "i18n_pieces", "i18n_gram", "i18n_ml", "i18n_mg" ]);
var EP_LANGUAGE = new Enum([ "en", "nl" ]);
var EP_BLOODTYPE = new Enum([ "A", "A_POSITIEF", "B", "O" ]);
var EP_ALGEMEEN_TYPE = new Enum([ "algemeen_notitie", "algemeen_stemming", "algemeen_zwangerschap", "algemeen_menstruatiecyclus",
		"algemeen_genotsmiddelen", "algemeen_ziekte", "algemeen_slaapritme", "algemeen_bioritme" ]);
var EP_MULTI_DAY = "epMultidayEvents";
var EP_MEDICINE_TYPE = new Enum([ "anti-epileptica, algemeen" ]);
var EP_DAY_PART = new Enum([ "morning", "afternoon", "evening", "night" ]);
var EP_BEHANDELSCHEMA_DAYPARTS = new Enum([ "08:00", "12:00", "18:00", "22:00" ]);
var EP_QUICKDIALS_MAX = 3;
var EP_PASSWORD_SALT = '69046c51f043758d1662370952e9b62f'; //random

/**
 * Genereert een nieuw uniek ID om een object te identificeren. Gebruikt nu unix
 * epoch milliseconds.
 */
function uniqueId() {
	var newDate = new Date;
	return newDate.getTime();
}

/**
 * Use this as a base class for all data containing objects.
 * 
 * @param objectType
 *            Defines the internal type.
 * 
 * @returns {epBase}
 */
epBase = function(objectType) {
	this.objectType = objectType;
	this.id = uniqueId();
	
	/*
	 * 29-03-2013: Deze property toegevoegd zodat het in de toekomst mogelijk is om te controleren
	 * met welke versie van het datamodel een epXXX object is opgeslagen zodat evt datafixes makkelijker
	 * en gerichter zijn door te voeren.
	 */
	this.dataModelVersion = EP_BUILD;
};

/**
 * CONTACT defines the persons that can be defined by the owner so they can be
 * used as a quickdial contact or as a recipient for the reports.
 */
epContact = function(contactName) {
	$.extend(this, new epBase("epContact"));

	var name;
	var phoneNumber;
	var email;
	var role;
	var isQuickDial;
	var isSmsContact;
	var isFollower;
	// constructor
	this.name = contactName;

};

/**
 * BIO beschrijft informatie over de gebruiker van de applicatie. Gegevens die
 * nodig zijn voor het functioneren, of die standaard waarden overschrijven.
 */
epBio = function() {
	$.extend(this, new epBase("epBio"));

	var name;
	var birthdate;
	var email;
	var phonenumber;
	var customAlarmMessage;
};

/**
 * ACCOUNT bevat de inloggegevens voor de server.
 */
epAccount = function() {
	$.extend(this, new epBase("epAccount"));
	
	var username;
	var password;
	var loggedIn;
};

/**
 * SETTING object. Gebruik voor globale settings. Wordt initieel gevuld met
 * default settings.
 * 
 * @param name
 * @param value
 */
epSettings = function () {
    $.extend(this, new epBase("epSettings"));

    this.language = '';
    this.build = EP_BUILD;
    this.pincode = '';
    this.key;
    this.medicinelist = '';
    this.askPin = false;
    this.useMyData;
    this.adminMail;
};

epOmniture = function() {
	$.extend(this, new epBase("epOmniture"));
	this.omnitureUdid = GUID();
	this.launchCount = 0;
	this.alarmCount = 0;
	this.agendaCount = 0;
};


epEventCustomMedicine = function(startDateTime, medicine) {
	$.extend(this, new epEvent(new epEventType("AfwijkendMedicijngebruik", false), startDateTime));
	this.drugName = medicine ? medicine.drugName : "";
	this.time;
 	this.unit;
 	this.quantity;
 	this.daypart;
 	this.reason;
 	this.behandelschemaServerId;
};
 

epArtsBehandelSchema = function() {
   $.extend(this, new epBase("epArtsBehandelSchema"));
	this.dateCreated = new Date();
	this.medicijnSchemas = [];
};

epBehandelSchema = function(medicineName, serverId) {
   $.extend(this, new epBase("epBehandelSchema"));
	this.startDate = new Date();
   this.drugName = medicineName;
	this.unit;
	this.serverId = serverId;
	this.sendNotification = false;
	this.innameMomenten = [];
};

epInnameMoment = function(date, quantity) {
   $.extend(this, new epBase("epInnameMoment"));
	this.date = date;
	this.quantity = quantity;
};

epRecurringMedicine = function(medicine) {
	$.extend(this, new epBase("epRecurringMedicine"));
	// this.drugId = medicine.id; //todo: later
	this.drugName;
	this.unit;
	this.dayparts = [];
};

epMedicineDayPart = function(l_time_of_day, l_time, l_quantity) {

	this.daypart = l_time_of_day || "morning";
	this.quantity = l_quantity || 1;
	this.time = l_time || getDefaultTime(this.daypart);

	function getDefaultTime(time_of_day) {
		switch (time_of_day) {
		case "morning":
			return "08:00";
			break;
		case "afternoon":
			return "12:00";
			break;
		case "evening":
			return "18:00";
			break;
		case "night":
			return "22:00";
			break;
		default:
			return new Date().toDisplayTimeString();
			break;
		}
	}
};

/**
 * EVENT. Abstracte base class voor specifieke gebeurtenissen.
 */
epEvent = function(eventType, startDateTime) {
	$.extend(this, new epBase("epEvent"));
	this.eventType = eventType;
	this.startDateTime = startDateTime;
	this.eindDateTime;
	this.notes = "";
	this.isEvent = true;
};

/**
 * epEvent instances can be of a specific type. EventTypes can be provided by
 * default by the application or they can be created by the user.
 * 
 * @param typeName
 *            De naam van het event, bijvoorbeeld 'Alcoholgebruik'.
 * @param custom
 *            Wanneer true, geeft dit aan als het een door de gebruiker
 *            aangemaakt type is
 * @returns {epEventType}
 */
epEventType = function(typeName, custom) {
	$.extend(this, new epBase("epEventType"));

	this.custom = custom;
	this.eigenTitel = "";
	this.name = typeName;

	/*
	 * Om bij te houden wanneer dit eventType voor het laatst is gebruikt. Bij
	 * het aanmaken by default op 'nu' zetten.
	 */
	var lastUsed = uniqueId();
};

/**
 * Wordt gebruikt om het type van de aanval vast te leggen.
 * 
 * @param typeName
 * @returns {epAanvalType}
 */
epAanvalType = function(typeName) {
	$.extend(this, new epBase("epAanvalType"));

	this.typeName = typeName;
	this.description = "";
};

epEventAlgemeen = function(startDateTime, algemeenType) {
	$.extend(this, new epEvent(new epEventType(algemeenType, false), startDateTime));
};

/**
 * EVENTSTROKE. Subclass of EVENT
 * 
 * @param startDate
 *            The date at which the event occurs
 * @param strokeType
 *            The type of event
 * 
 * @returns {epEventStroke}
 */
epEventStroke = function(startDateTime, strokeType) {
	$.extend(this, new epEvent(new epEventType("Aanval", false), startDateTime));

	this.strokeType = strokeType;

	var intensity = 0;
	var strokeCount = 1;
	var duration = 0;
};

/**
 * EVENTINCIDENTDRUG. Subclass of EVENT
 * 
 * @param startDate
 * @param drugName
 * @returns {epEventIncidentDrug}
 */
epEventIncidentDrug = function(startDateTime, drugName) {
	$.extend(this, new epEvent(new epEventType("IncidenteleMedicatie", false), startDateTime));
	this.drugName = drugName;

	var quantity = 0;
	var unit; // Constant defined by EPEVENTINCIDENTDRUG_UNIT
};

epEventCustom = function(startDateTime, customEventType) {
	$.extend(this, new epEvent(customEventType, startDateTime));
	var endTime;
};

epEventSideEffect = function (startDateTime, name) {
    $.extend(this, new epEvent(new epEventType("Bijwerking", false), startDateTime));
    this.name= name;
};

/*-
 * EPGEO. Subclass of epBase, used to keep track of a range of gps coordinates.
 * The coordinates field is an array that should look like this: 
 * 
 * [ [longitude, latitude, elevation, timestamp]
 * 
 * The last 2 fields, elevation and timestamp are optional
 * 
 * E.g.
 * 
 * [[6.5246331, 53.1978617, 3, 1328537936 ], [6.527507928, 53.199799540896, 7, 1328538465 ], [etc,etc] ]
 * 
 * Dit is ook het JSON object dat wordt opgestuurd naar de server.
 * 
 */
epGeo = function() {
	$.extend(this, new epBase("epGeo"));
	this.accuracy;
	this.followmesessionid;
	this.points = new Array();
};

epPoint = function(date, lat, lon) {
	this.timestamp = date.getTime();
	this.latitude = lat;
	this.longitude = lon;
};