/**
 * GPS tracker die via de localstorage een lijstje met coordinaten bij houdt.
 * 
 */

var gpsTracker = {

	geo : null,
	
	lastSignal : 0,
	hasSignal : false,
	
	getGeo : function() {
		var geoArr = getJSON("epGeo");
		if (geoArr == null) {
			this.geo = new epGeo();
			// coords = this.geo.points;
			storeJSON(this.geo);
		} else {
			this.geo = geoArr[0];
		}
		
		return this.geo;
	},
	/**
	 * Deze functie wordt aangeroepen als er nieuwe gps coordinaten zijn
	 * binnengekomen. Deze worden bijgewerkt in het epGeo object in de local
	 * storage.
	 * 
	 * @param position
	 */
	onNewCoordinates : function(position) {
		var geo = gpsTracker.getGeo();

		var lat = position.coords.latitude;
		var lon = position.coords.longitude;
		var accuracy = position.coords.accuracy;
				
// console.log("gpsTracker: received new coordinates, lat: " + lat + " + lon: " + lon);
// var dateLow = new Date();
// console.log(debugSeconds);
// debugText = dateLow.toString("hh:mm:ss") + " -- acc: " + accuracy + "m";
// signalText = " -- last gps signal: " + debugSeconds + "s ago.";
// console.log(debugText);
// gpsTracker.setDebugText();
// }
		
		updateIkOpDeKaart(lat, lon);
		gpsTracker.hasSignal = true;
		geo.accuracy = accuracy;
		if (accuracy <= EP_GPS_MIN_ACCURACY_METERS || accuracy == null) {
			var date = new Date();
			geo.points.push(new epPoint(date, lat, lon));
			storeJSON(geo, geo.id);
			startCounting();
		}
	},
	
	/**
	 * Wordt aangeroepen wanneer navigator.geolocation.watchPosition een fout
	 * geeft.
	 * 
	 * @param error
	 *            De foutmelding
	 */
	onError : function(error) {
		gpsTracker.hasSignal = false;
	},
    
    pollIphoneLocation : function() {
        console.log("pollIphoneLocation, request new coordinates");
        navigator.geolocation.getCurrentPosition( gpsTracker.onNewCoordinates, gpsTracker.onError, {
            enableHighAccuracy : true,
            maximumAge: 4000, // should be default, just in case
            timeout: 5000
		});
    },

	/*
	 * Aanmelden bij de watchPosition functie die bij nieuwe coordinaten ene
	 * aanroep van de gpsTracker doet.
	 */
	start : function(){
		watchID = navigator.geolocation.watchPosition(gpsTracker.onNewCoordinates, gpsTracker.onError, {
		enableHighAccuracy : true,
		maximumAge: 4000, // should be default, just in case
		timeout: 5000
		});
		startCounting();
	},
	
	stop : function(){
		navigator.geolocation.clearWatch(watchID);
		gpsTracker.clearPoints();
		gpsTracker.hasSignal = false;
		stopCounting();
	},
	
	clearPoints : function() {
		var geo = gpsTracker.getGeo();
		geo.points = [];
		storeJSON(geo, geo.id); 
	},
	
};

/**
 * Keep the phone screen on while this functionality is running.
 */
function lockScreen() {
	if (window.plugins != undefined && window.plugins.PowerManagement != undefined) {
		window.plugins.PowerManagement.acquire(function() {
			console.log('lock acquired');
		}, function() {
			console.log('no lock acquired');
		});
	} else {
		console.log('Cannot prevent screen from locking: no powermanagement available');
	}
}

/**
 * Update de google maps link.
 */
function updateIkOpDeKaart(lat, lon) {
	var link = [ 'http://maps.google.com/?q=loc:', lat, ",", lon ];
	if ($('#ikOpDeKaartLink').length) {
		$('#ikOpDeKaartLink').attr("href", link.join(''));
	}
}

var watchID = null;

var signalText = "";
var debugText = "";
// var debugSeconds = 0;
var debugCounting = null;

function startCounting() {
	clearInterval(debugCounting);
	gpsTracker.lastSignal = 0;
    debugCounting = setInterval(function () {
        gpsTracker.lastSignal++;
    }, 1000);
}

function stopCounting() {
    clearInterval(debugCounting);
}