var APPLICATION_LANGUAGE;

/**
 * Deze methode wordt door de controllers gebruikt om inheritance te gebruiken.
 * 
 * Zie voor details methode 8 uit het boek Object Oriented Javascript: Using a
 * Mix of Prototypal Inheritance and Copying Properties.
 * 
 * Deze methode zorgt ervoor dat child objecten een bestaande function kunnen
 * extenden dmv 'this._super.SomeFunction( someArgs )'. Tevens hebben alle child
 * objects toegang tot alle properties van de parent.
 * 
 * Usage: var myChild = epExtend( myParent, { vul hier de child met 'stuff' });
 * 
 * @param o
 *            Parent object van waaruit de eigenschappen moeten worden
 *            overgenomen
 * @param newFunctionality
 *            De nieuwe functionaliteit voor het child object
 * 
 * @returns Geeft een volledig geinitializeerd child object terug.
 */
function epExtend(parentObject, newFunctionality) {
	var resultObject;

	/*
	 * Zorg voor een prototype, zonder dat een functie van het kind de
	 * functionaliteit van de parent overschrijft.
	 */
	function F() {
	}
	F.prototype = parentObject;
	resultObject = new F();

	/*
	 * Zorg ervoor dat de parent beschikbaar is via de '_super' property.
	 */
	resultObject._super = parentObject;

	/*
	 * Kopieer alle properties naar het child object. Primitives worden
	 * letterlijk gekopieerd. Functie by reference.
	 */
	for ( var i in newFunctionality) {
		resultObject[i] = newFunctionality[i];
	}

	return resultObject;
}

/**
 * Een one-way hash functie die van een gegeven input altijd dezelfde output
 * geeft. Resultaten verschillen afhankelijk van de meegegeven 'salt'.
 * 
 * @see http://nl.wikipedia.org/wiki/Salt_(cryptografie)
 * 
 * @param input
 *            De input die versleuteld moet worden.
 * @param salt
 *            De salt, mag per device verschillen, maar moet op 1 toestel ALTIJD
 *            gelijk zijn. Omdat anders de resultaten van deze one-way hash niet
 *            te vergelijken zijn.
 * @returns Een one-way hash resultaat als string.
 */
function oneWayHash(input) {
	var inputArr = GibberishAES.s2a(oneWayKey(input));
	return GibberishAES.a2h(GibberishAES.Hash.MD5(inputArr));
}

function oneWayKey(input) {
	var uuid = '';
	if (window.device != undefined) {
		uuid = window.device.uuid;
	} else {
		uuid = "12345678";
	}

	return (input + uuid + wortel(4));
}

function GUID ()
{
    var S4 = function ()
    {
        return Math.floor(
                          Math.random() * 0x10000 /* 65536 */
                          ).toString(16);
    };
    
    return (
            S4() + S4() + "-" +
            S4() + "-" +
            S4() + "-" +
            S4() + "-" +
            S4() + S4() + S4()
            );
}

function wortel(arg) {
	return ((arg * 5) + 3);
}

var EPP_HEADER_FOOTER_HEIGHT = 40 + 62 + 50; // 50 = magic number om de
// scrollbar te laten verdwijnen
function getContentHeight() {
	// var header =$('.eppHeader').outerHeight();
	// var footer = $('.navbar-epp').outerHeight();

	return ($(window).height() - EPP_HEADER_FOOTER_HEIGHT);
}

// load I18N bundles
function loadBundles(lang) {
	APPLICATION_LANGUAGE = lang;

	if (APPLICATION_LANGUAGE != 'nl' && APPLICATION_LANGUAGE != 'en') {
		APPLICATION_LANGUAGE = 'nl';
	}

	jQuery.i18n.properties({
		cache : true,
		name : 'Messages',
		path : 'bundle/',
		mode : 'map',
		language : APPLICATION_LANGUAGE,
		callback : function() {
			translateAllElements();
		}
	});
}



function initializeCalendar() {
	/*
	 * Globale variable voor de hoogte van calender items.
	 */
	var calendar_header_height = 24; // incl spacing/padding, better to read
	var week_item_spacing = 12;
	EP_calendar_month_availableHeightPixels = (getContentHeight() - calendar_header_height);
	EP_calendar_week_itemHeight = Math.floor((getContentHeight()) / 4) - week_item_spacing;

	$.ajax({
		url : 'js/libs/date-' + APPLICATION_LANGUAGE + '.js',
		dataType : "script",
		cache : true,
		success : function() {
			console.log('date language extensions geladen.');
			setWeek();
			setMonth();
			redrawCalendar();
		}
	});
}

/**
 * Voeg een grid icoontje aan de rechterkant van een inputfield toe. Let op dat
 * deze functie in een .on("pageinit", function() {} ) wordt aangeroepen, anders
 * zet jQueryMobile er weer eigen schaduw effecten etc in je inputField.
 * 
 * Zet na je input veld een hyperlink als een placeholder voor het link
 * icoontje: <a href="#" id="inputLink">Alt text</a>
 * 
 * @param inputField
 *            Het textfield waar het icon aan de rechterkant moet komen
 * @param inputLink
 *            Het id van de hyperlink die na het input veld staat
 * @param onClickFunction
 *            De functie die moet worden aangeroepen wanneer op het icoontje
 *            wordt geklikt.
 */
function createIconLink(inputField, inputLink, onClickFunction) {
	var focus = inputField.wrap('<div class="ui-input-datebox ui-shadow-inset ui-corner-all ui-body-c"></div>').parent();

	inputLink.bind('vclick', onClickFunction).buttonMarkup({
		icon : 'grid',
		iconpos : 'notext',
		corners : true,
		shadow : true
	}).appendTo(focus).css({
		'vertical-align' : 'middle',
		'float' : 'right'
	});
	inputField.removeClass('ui-corner-all ui-shadow-inset').focus(function() {
		focus.addClass('ui-focus');
		inputField.removeClass('ui-focus');
	}).blur(function() {
		focus.removeClass('ui-focus');
		inputField.removeClass('ui-focus');
	});
}
function renderFooter() {

	document.write('<div class="navbar-epp" data-role="footer" >');
	document.write('<div class="navbar-epp" data-role="navbar">');
	document.write('<ul>');
	document.write(' <li><div class="navitem"><a href="#" id="navbarCalendarLink" data-icon="navbar-calendar" epp-page="#calendarWeekPage" ><label class="i18n_calendar"></label></a></div></li>');
//	document.write(' <li><div class="navitem"><a href="#" id="navbarReportLink" data-icon="navbar-report" epp-page="#reportPage" ><label class="i18n_reporting"></label></a></div></li>');
	document.write(' <li><div class="navitem"><a href="#" id="navbarSettingsLink" data-icon="navbar-settings" epp-page="#settingsPage" ><label class="i18n_settings"></label></a></div></li>');
	document.write(' <li><div class="navitem"><a href="#" id="navbarHelpLink" data-icon="navbar-help" epp-page="#helpScherm" ><label class="i18n_help"></label></a></div></li>');
	document.write(' <li><div class="navitem"><a href="#" id="navbarAlarmLink" data-icon="navbar-alarm" epp-page="#alarmScherm" ><label class="i18n_alarm"></label></a></div></li>');
	document.write('</ul>');
	document.write('</div>');
	document.write('</div>');
}

String.prototype.format = function() {
	var args = arguments;
	return this.replace(/{(\d+)}/g, function(match, number) {
		return typeof args[number] != 'undefined' ? args[number] : match;
	});
};

String.prototype.hasValue = function() {
	return (this != undefined && this != "");
};

Date.prototype.toDisplayDateString = function() {
	return this.toString(jQuery.i18n.prop('i18n_textbox_dateformat'));
};

Date.prototype.toDisplayTimeString = function() {
	return this.toString('HH:mm');
};

/**
 * Overriden van het resultaat van JSON.stringify( date );
 * 
 * Het normale formaat geeft ook .000Z als extentie, dit leidt tot problemen als
 * dit op een mobiele telefoon moet worden geparsed.
 * 
 * @param key
 *            See super
 * @returns {String} See super
 */
Date.prototype.toJSON = function(key) {
	return this.toString("yyyy-MM-ddTHH:mm:ss");
};

function translateAllElements() {
	// Translate all elements that have class='i18n_ prefix
	$('[class^="i18n"]').each(function(i) {
		var elementClass = $(this).attr("class").match(/i18n[^\s]+/g);
		$(this).html(jQuery.i18n.prop(elementClass));
		// console.log(jQuery.i18n.prop(elementClass));
	});
}

/**
 * Haalt de huidige locatie op.
 */
var getLocation = function() {
	var success = function(loc) {
		return function() {
			this.latitude = loc.coords.latitude;
			this.longitude = loc.coords.longitude;
		};
	};
	var failure = function() {
		return null;
	};

	navigator.geolocation.getCurrentPosition(success, failure, {
		maximumAge : 3000,
		timeout : 5000,
		enableHighAccuracy : true
	});
};

function Enum(constantsList) {
	for ( var i in constantsList) {
		this[constantsList[i]] = i;
	}
}

getEnumByValue = function(my_enum, value) {
	for ( var k in my_enum) {
		if (my_enum.hasOwnProperty(k)) {
			if (my_enum[k] == value) {
				return k;
			}
		}
	}
	return undefined;
};

function initializeLogin(setupLanguage, druglist) {
	var settings = getJSON('epSettings');

	if (settings == null || settings == undefined) {
		settings = new epSettings();
		settings.language = setupLanguage;
		settings.medicinelist = druglist;
		storeJSON(settings);

		// Stel de gekozen taal in
		loadBundles(setupLanguage);
	} else {
		console.log(settings[0].language);
		loadBundles(settings[0].language);
	}
}

function initializeApplication(reloadBundles) {
	// nieuwe pagina, opnieuw language bundles laden
	if (reloadBundles == true)
		loadBundles(getSettings().language);

	// kijk of er al aanvaltypes zijn, zo niet: 1 default aanmaken
	createStandaardAanval();

	// initializeer de multiday cache
	EP_MULTIDAY_CACHE = getJSON(EP_MULTI_DAY);

	// chrome has a problem with scrollview. Disable scrollview for chrome. In
	// iOs all is fine.
	var agent = navigator.userAgent.toLowerCase();
	if (agent.match(/android/) || agent.match(/iphone/) || agent.match(/ipad/)) {
		useScrollViewOnUiContent = true;
	} else {
		useScrollViewOnUiContent = false;
	}
}

var sortBy = function(field, reverse, primer) {
	reverse = (reverse) ? -1 : 1;
	return function(a, b) {
		a = a[field];
		b = b[field];

		if (typeof (primer) != 'undefined') {
			a = primer(a);
			b = primer(b);
		}

		if (a < b)
			return reverse * -1;
		if (a > b)
			return reverse * 1;
		return 0;
	};
};


/** 
    Alternatief voor alert functie. IS NON_BLOCKING. Om code na de alert uit te voeren de onComplete callback meegeven
*/
function phoneAlert(message,onComplete) {
    if (typeof navigator.notification === "undefined") {
        if (window.alert(message)) {
            if (onComplete !== null) {
                onComplete();    
            }
        }
    } else {
        navigator.notification.alert(message, onComplete, "", "OK");
    }
}

function phoneConfirm(message,onConfirm, onReject, title) {
    //do a regular confirm if not on device
    if (typeof navigator.notification === "undefined") {
        if (window.confirm(message)) {
            if (onConfirm !== null) {
                onConfirm();    
            }
        }
        else if (onReject !== null) {
            onReject();
        }
    } else {
        var bevestiging = (title == undefined ? $.i18n.prop('i18n_confirm') : title);
        var ok = "OK";
        var cancel = $.i18n.prop('i18n_cancel');
        var callBack = function (button) {
            if (button === 1) {
                onConfirm();
            } else {
                onReject();
            }
        };
        navigator.notification.confirm(message, callBack, bevestiging, ok + "," + cancel);
    }
}
/**
 * Laadt een json file in en stopt deze in een list(view)
 * 
 * @param fileName
 * @param targetList
 *            ID
 */
function loadJSONFileToList(fileName, targetList) {

	$.getJSON(fileName, function(data) {
		// clear the list
		$(targetList).html("");

		var items = [];
		var currentDividerCode = '-1'; // Ascii code for 'A'
		var title = null;
		var subtitle = null;

		for ( var i = 0; i < data.length; i++) {
			var subtitle = data[i].naam;

			// Als de lijst alleen maar 'naam' elementen bevat is dit ook de
			// titel. Zo niet, gebruik dan de titel
			if (data[i].title != undefined)
				title = data[i].title;
			else
				title = data[i].naam;

			var currentChar = title.charAt(0).toUpperCase();
			var currentCharCode = currentChar.charCodeAt(0);

			/*
			 * Initialize charCode op eerste item in de lijst. Aanname: de lijst
			 * is gesorteerd op alfabet. Bij een nieuwe letter, eerst een
			 * divider in de lijst stoppen.
			 */
			if (currentDividerCode != currentCharCode) {
				currentDividerCode = currentCharCode;
				items.push('<li data-role="list-divider">', currentChar, '</li>');
			}

			items.push('<li><h4>', title, '</h4>');
			if (data[i].title != undefined)
				items.push('<p>', subtitle, '</p>');

			items.push('</li>');
		}

		$(targetList).append(items.join(""));
	});
}

/**
 * Retourneert het verschil in dagen tussen twee datums. Eerste datum is de
 * vroegste.
 * 
 * @param date1
 * @param date2
 * @param roundDown afronden naar beneden (als de tijd verschilt)
 * @returns verschil in dagen
 */
function getDifferenceInDays(date1, date2, roundDown) {
		return (Math.ceil((date2.getTime() - date1.getTime()) / (ONE_DAY_IN_MILLISECONDS)));
}
var ONE_DAY_IN_MILLISECONDS = 86400000; //1000 * 60 * 60 * 24;

