var $NO_DATA_FOUND;

$(function () {
    $("#reportPage").on("pagebeforeshow", function () {
        //vul een default starttijd/eindtijd in voor period zodat er niet geen datum kan staan.
        var today = new Date();
        var lastMonth = new Date().addMonths(-1);

        $("#reportEindDatum").val(today.toDisplayDateString());
        $("#reportBeginDatum").val(lastMonth.toDisplayDateString());

        buildReportItemsControlGroup();
    });

    $("#createReportSubmit").on("click", function () {
        var selectedRange = $('#reportRange > option:selected').val();
        /*
        * Einddatum is altijd vandaag, tenzij de rangeselector is gekozen.
        * Begindatum wordt bepaald aan de hand van de gekozen range (en
        * einddatum). Tenzij de rangeselector is gekozen.
        */
        var endDate = (selectedRange != "range") ? new Date() : Date.parse($('#reportEindDatum').val());
        var beginDate = (selectedRange != "range") ? determineBeginDate(selectedRange, endDate) : Date.parse($('#reportBeginDatum').val());

        if (selectedRange === "range") {
            if (!validator.validateStarttimePreceedsEndTime(beginDate, endDate)) {
                return false;
            }
        }
        if (!validator.validateAtLeastOneCheckboxIsChecked('#reportPage')) {
        	return false;
        }
        
        var report = createHTMLReport(beginDate, endDate);
        var bio = getJSON('epBio');
        var name = (bio != null) ? bio[0].name || "" : "";
        var subject = jQuery.i18n.prop('i18n_report_title', name, beginDate.toDisplayDateString(), endDate.toDisplayDateString());

        QuintorPhonegapPlugins.sendEmail(subject, report);
    });

    $("#showChart").on("click", function () {

        var selectedRange = $('#reportRange > option:selected').val();
        var endDate = (selectedRange != "range") ? new Date() : Date.parse($('#reportEindDatum').val());
        var beginDate = (selectedRange != "range") ? determineBeginDate(selectedRange, endDate) : Date.parse($('#reportBeginDatum').val());

    	if (selectedRange === "range") {
            if (!validator.validateStarttimePreceedsEndTime(beginDate, endDate)) {
                return false;
            }
        }
    	
        if (!validator.validateAtLeastOneCheckboxIsChecked('#reportPage')) {
        	return false;
        }
        navigationController.gotoPage('#plotPage');
        drawPlot();
    });


    $("#reportRange").on("change", function () {
        var selectedOption = $(this).find('option:selected').val();
        if (selectedOption == "range") {
            $('#reportDateRange').show();
        } else {
            $('#reportDateRange').hide();
        }
    });

});

function determineBeginDate(range, endDate) {
	endDate = endDate || Date.now();
	var beginDate = new Date(endDate);
	switch (range) {
	case "all":
		beginDate.addYears(-4); // TODO: remove the magic number
		break;
	case "range":
		// hier mag hij nooit komen
		break;
	default:
		var number = range.charAt(0);
		var month_or_week = range.charAt(1);
		if (month_or_week == 'm') {
			beginDate.addMonths(-number);
		} else if (month_or_week == 'w') {
			beginDate.addWeeks(-number);
		}
		break;
	}

	return beginDate;
}

/**
 * Maak een selecteerbare lijst met checkboxen zodat je events kan selecteren
 * die in het report moeten.
 */
function buildReportItemsControlGroup() {
	var target = $('#reportItems > fieldset');
	target.empty();
	
	var $wrap = $('<div>');
	var legend = $('<legend />').html(jQuery.i18n.prop('i18n_report_events_to_include'));
	$wrap.append(legend);

	addReportCheckbox('Aanval', $wrap);
	addReportCheckbox('IncidenteleMedicatie', $wrap);
	addReportCheckbox('Medicijngebruik', $wrap);
	addReportCheckbox('CustomEvent', $wrap);
	addReportCheckbox('Bijwerking', $wrap);

	for (algemeenType in EP_ALGEMEEN_TYPE) {
		addReportCheckbox(algemeenType, $wrap);
	}
	
	target.append($wrap.html()).trigger('create');
}

function addReportCheckbox(cbName, wrap) {
	var input = $('<input />').attr( {
			name : cbName,
			id : cbName,
			checked : cbName == 'Aanval' ? true : false,
			type : 'checkbox',
			'class' : 'custom'	});
	var label = $('<label />').attr('for', cbName).html(jQuery.i18n.prop(cbName));
	wrap.append(input, label);
}

/**
 * Bouwt het volledige HTML overzicht op, op basis van begin en eindtijd.
 * 
 * @param beginDate
 * @param endDate
 * @returns
 */
function createHTMLReport(beginDate, endDate) {

    $NO_DATA_FOUND = $('<p>').html( jQuery.i18n.prop('i18n_no_data_found') );
	var totalDays = getDifferenceInDays(beginDate, endDate);
	var html = [];
	// var day = beginDate;
	var lastDay = (new Date(endDate)).addDays(-1);
	var $wrap = $('<div>').attr('id', 'tableWrap');
	$wrap.append(jQuery.i18n.prop('i18n_report_aanhef'));
	$wrap.append($('<br>'));
	$wrap.append(jQuery.i18n.prop('i18n_report_periode', beginDate.toString(jQuery.i18n.prop('i18n_textbox_dateformat')), lastDay
			.toString(jQuery.i18n.prop('i18n_textbox_dateformat')), totalDays));
	$wrap.append($('<br>'));
	createMedicijnOverzicht($wrap, beginDate, totalDays);
	$wrap.append($('<br>'));
	createIncidenteelMedicijnOverzicht($wrap, beginDate, totalDays);
	$wrap.append($('<br>'));
	createAanvalOverzicht($wrap, beginDate, totalDays);
	$wrap.append($('<br>'));
	createBijwerkingenOverzicht($wrap, beginDate, totalDays);
	$wrap.append($('<br>'));
	createGebeurtenissenOverzicht($wrap, beginDate, totalDays);
	$wrap.append($('<br>'));
	$wrap.append(jQuery.i18n.prop('i18n_report_disclaimer'));
	return $wrap.html();
}

/**
 * Medicijnenoverzicht
 */
function createMedicijnOverzicht($wrap, day, totalDays) {
	if (isSelected("Medicijngebruik")) {
		$wrap.append($('<h3>').html(jQuery.i18n.prop('i18n_report_medicijnen_overzicht')));
		var $medicineTable = addTable('medicineTable');
		addTableHeader($medicineTable, "i18n_name", "i18n_schedule");
		var medicines = mijnMedicatieController.getAll();
		for (i in medicines) {
			var med = medicines[i];
			addTableRow($medicineTable, med.drugName, getMedicineScheduleText(med));
		}
		if ($medicineTable.find('td').length) {
			$wrap.append($medicineTable);
		} else {
			$wrap.append($NO_DATA_FOUND.clone());
		}
	}
}

function getMedicineScheduleText(med) {
	var description = [];
	for (p in med.dayparts) {
		var part = med.dayparts[p];
		description.push(" ", jQuery.i18n.prop('i18n_medicine_scheme_description_units_time', part.quantity, part.time));
	}
	return description.join('').slice(0, -1); /*
												 * slice voor verwijderen
												 * laatste komma.
												 */
}

/**
 * Incidentele medicatie overzicht. Werkt op dezelfde manier als de overzichten
 * per gebeurtenis hieronder, alleen is de key hier medicijnnaam. Zo kan je zien
 * de hoeveelheden medicijnen die geslikt zijn.
 */
function createIncidenteelMedicijnOverzicht($wrap, day, totalDays) {
	if (isSelected("IncidenteleMedicatie") || isSelected("Medicijngebruik")) {
		$wrap.append($('<h3>').html(jQuery.i18n.prop('i18n_report_incidentele_medicijnen_overzicht')));
		var $incMedTable = addTable('incMedTable');
		addTableHeader($incMedTable, "i18n_name", "i18n_qnt");
		var incMedStats = new Array();
		for ( var d = 0; d < totalDays; d++) {
			var events = getJSON(day.toDateString());
			if (events != null) {
				for ( var e in events) {
					var event = events[e];
					if (event.eventType.name === "IncidenteleMedicatie") {
						if (incMedStats[event.drugName]) {
							incMedStats[event.drugName] += 1;
						} else {
							incMedStats[event.drugName] = 1;
						}
					}
				}
			}
			day.addDays(1);
		}
		day.addDays(-totalDays); // reset the day counter.

		for ( var s in incMedStats) {
			addTableRow($incMedTable, s, incMedStats[s]);
		}

		if ($incMedTable.find('td').length) {
			$wrap.append($incMedTable);
		} else {
			$wrap.append($NO_DATA_FOUND.clone());
		}
	}
}

/**
 * Aanvallenoverzicht
 * 
 * @param $wrap
 *            element waarin de tabel gewrapt moet worden
 */
function createAanvalOverzicht($wrap, day, totalDays) {
	if (isSelected("Aanval")) {
		$wrap.append($('<h3>').html(jQuery.i18n.prop('i18n_report_aanvallen_overzicht')));
		var $aanvalTable = addTable('aanvalTable');
		addTableHeader($aanvalTable, "i18n_date", "i18n_attack_type", "i18n_intensity", "i18n_duration", "i18n_qnt", "i18n_notes");
		for ( var d = 0; d < totalDays; d++) {
			var events = getJSON(day.toDateString());
			if (events != null) {
				for ( var e in events) {
					var event = events[e];
					if (event.eventType.name === "Aanval") {
						addTableRow($aanvalTable, event.startDateTime.toString('yyyy/MM/dd hh:mm'), event.strokeType, event.intensity,
								event.duration, event.strokeCount, event.notes);
					}
				}
			}
			day.addDays(1);
		}
		day.addDays(-totalDays); // reset the day counter.

		if ($aanvalTable.find('td').length) {
			$wrap.append($aanvalTable);
		} else {
			$wrap.append($NO_DATA_FOUND.clone());
		}
	}
}

/**
* Aanvallenoverzicht
* 
* @param $wrap
*            element waarin de tabel gewrapt moet worden
*/
function createBijwerkingenOverzicht($wrap, day, totalDays) {
    if (isSelected("Bijwerking")) {
        $wrap.append($('<h3>').html(jQuery.i18n.prop('i18n_report_bijwerkingen_overzicht')));
        var $targetTable = addTable('bijwerkingTable');
        addTableHeader($targetTable, "i18n_date", "i18n_side_effect_name", "i18n_intensity_side_effect", "i18n_notes");
        for (var d = 0; d < totalDays; d++) {
            var events = getJSON(day.toDateString());
            if (events != null) {
                for (var e in events) {
                    var event = events[e];
                    if (event.eventType.name === "Bijwerking") {
                        addTableRow($targetTable, event.startDateTime.toString('yyyy/MM/dd hh:mm'), event.name, event.intensity, event.notes);
                    }
                }
            }
            day.addDays(1);
        }
        day.addDays(-totalDays); // reset the day counter.

        if ($targetTable.find('td').length) {
            $wrap.append($targetTable);
        } else {
			$wrap.append($NO_DATA_FOUND.clone());
		}
    }
}


/**
 * Gebeurtenissen overzicht: we lopen door alle gebeurtenissen heen en hashen op
 * gebeurtenis naam (hogen de waarde telkens met 1 op als we een zelfde event
 * tegenkomen).
 */
function createGebeurtenissenOverzicht($wrap, day, totalDays) {
	var drawTable = false;
	$wrap.append($('<h3>').html(jQuery.i18n.prop('i18n_report_gebeurtenissen_overzicht')));
	var $eventTable = addTable('eventTable');
	addTableHeader($eventTable, "i18n_name", "i18n_qnt");
	var eventStats = new Array();
	for ( var d = 0; d < totalDays; d++) {
		var events = getJSON(day.toDateString());
		if (events != null) {
			for ( var e in events) {
				if (eventStats[events[e].eventType.name]) {
					eventStats[events[e].eventType.name] += 1;
				} else {
					eventStats[events[e].eventType.name] = 1;
				}
			}
		}
		day.addDays(1);
	}
	day.addDays(-totalDays); // reset the day counter.
	// TODO: ook nog voor de multidayevents...
	for ( var s in eventStats) {
		if (isSelected(s)) {
			addTableRow($eventTable, jQuery.i18n.prop(s), eventStats[s]);
		}
	}

	if ($eventTable.find('td').length) {
		$wrap.append($eventTable);
	} else {
		$wrap.append($NO_DATA_FOUND.clone());
	}
}

/**
 * Returns een nieuwe lege tabel
 * 
 * @param tableName
 * @returns
 */
function addTable(tableName) {
	return $('<table style="width: 100%;">').attr('id', tableName).css("border", "1px solid #eeeeee");
}

/**
 * Voegt een nieuwe rij toe aan een tabel. Variable argumentenlijst: de eerste
 * is altijd de tabelnaam. Daarna voor elke kolom een nieuwe parameter.
 * 
 * @param $tbl
 */
function addTableRow($tbl) {
	var $tr = $('<tr>');
	for ( var i = 1; i < addTableRow.arguments.length; i++) {
		$tr.append($('<td>').text(addTableRow.arguments[i]));
	}
	$tbl.append($tr);
}

/**
 * Voegt een header toe aan een tabel
 * 
 * @param $tbl
 */
function addTableHeader($tbl) {
	var $tr = $('<tr>');
	for ( var i = 1; i < addTableHeader.arguments.length; i++) {
		$tr.append($('<th style="text-align: left;">').text(jQuery.i18n.prop(addTableHeader.arguments[i])));
	}
	$tbl.append($tr);
}

/*
 * 
 */
function isSelected(type) {
	return $('#reportItems > fieldset > div > input[name="' + type + '"]').is(':checked');
}
